import requests
import os
from ..common import prettyllog
from ..gitea.git import clone_git_project
from ..netbox.netbox import get_netbox_inventory_from_tag


import pprint
import json

import pprint
import json 
import tempfile
import git




debug = True





import os
import requests

def login():
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    user = os.getenv('MANISBLE_SEMAPHORE_USER')
    password = os.getenv('MANISBLE_SEMAPHORE_PASSWORD')
    url = f"{baseurl}/api/auth/login"
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }
    data = {
        'auth': user,
        'password': password
    }

    # Create a session
    session = requests.Session()

    # Make the login request
    response = session.post(url, headers=headers, json=data)
    if response.status_code == 204:
        # Successful request
        prettyllog("semaphore", "Init", "login", "ok",  response.status_code , "login successful", severity="INFO")
        return session  # Return the session for subsequent requests
    else:
        # ERRORed request
        prettyllog("semaphore", "Init", "login", "error", response.status_code , "login failed", severity="ERROR")
        return None
    
def crete_user(session, user):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    user_url = f"{baseurl}/api/users"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.post(user_url, headers=headers, json=user)
    if response.status_code == 201:
        # Successful request
        prettyllog("semaphore", "create", user['username'], "ok", response.status_code , "create user", severity="INFO")
        return response.json()
    else:
        # Failed request
        prettyllog("semaphore", "create", user['username'], "error", response.status_code , "create user", severity="ERROR")

def get_user(session):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    user_url = f"{baseurl}/api/users"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.get(user_url, headers=headers)
    if response.status_code == 200:
        # Successful request
        users = response.json()
        # map users by username
        users_by_username = {}
        for user in users:
            users_by_username[user['username']] = user
            if debug:
                prettyllog("semaphore", "get", user['username'], "ok", response.status_code , "loadning users", severity="DEBUG")
        prettyllog("semaphore", "get", "user", "ok", response.status_code , "loadning users", severity="INFO")
        return users_by_username
    else:
        # Failed request
        prettyllog("semaphore", "get", "user", "error", response.status_code , "loadning users", severity="ERROR")

def create_team(session, team):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    team_url = f"{baseurl}/api/teams"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.post(team_url, headers=headers, json=team)
    if response.status_code == 201:
        # Successful request
        prettyllog("semaphore", "create", team['name'], "ok", response.status_code , "create team", severity="INFO")
        return response.json()
    else:
        # Failed request
        prettyllog("semaphore", "create", team['name'], "error", response.status_code , "create team", severity="ERROR")

def get_team(session):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    team_url = f"{baseurl}/api/teams"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.get(team_url, headers=headers)
    if response.status_code == 200:
        # Successful request
        teams = response.json()
        # map teams by name
        teams_by_name = {}
        for team in teams:
            teams_by_name[team['name']] = team
            if debug:
                prettyllog("semaphore", "get", team['name'], "ok", response.status_code , "loadning teams", severity="DEBUG")
        prettyllog("semaphore", "get", "team", "ok", response.status_code , "loadning teams", severity="INFO")
        return teams_by_name
    else:
        # Failed request
        prettyllog("semaphore", "get", "team", "error", response.status_code , "loadning teams", severity="ERROR")

def create_team_member(session, team_id, team_member):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    team_member_url = f"{baseurl}/api/team/{team_id}/members"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.post(team_member_url, headers=headers, json=team_member)
    if response.status_code == 201:
        # Successful request
        prettyllog("semaphore", "create", team_member['username'], "ok", response.status_code , "create team member", severity="INFO")
        return response.json()
    else:
        # Failed request
        prettyllog("semaphore", "create", team_member['username'], "error", response.status_code , "create team member", severity="ERROR")

def create_key(session, key):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    key_url = f"{baseurl}/api/keys"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.post(key_url, headers=headers, json=key)
    if response.status_code == 201:
        # Successful request
        prettyllog("semaphore", "create", key['name'], "ok", response.status_code , "create key", severity="INFO")
        return response.json()
    else:
        # Failed request
        prettyllog("semaphore", "create", key['name'], "error", response.status_code , "create key", severity="ERROR")

def create_repository(session, repository):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    repository_url = f"{baseurl}/api/repositories"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }
    pprint.pprint(repository)
    # Use the session for the request
    response = session.post(repository_url, headers=headers, json=repository)
    if response.status_code == 201:
        # Successful request
        prettyllog("semaphore", "create", repository['name'], "ok", response.status_code , "create repository", severity="INFO")
        return response.json()
    else:
        # Failed request
        prettyllog("semaphore", "create", repository['name'], "error", response.status_code , "create repository", severity="ERROR")

def get_repository(session):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    repository_url = f"{baseurl}/api/repositories"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }
    # Use the session for the request
    response = session.get(repository_url, headers=headers)
    if response.status_code == 200:
        # Successful request
        repositories = response.json()
        # map repositories by name
        repositories_by_name = {}
        for repository in repositories:
            repositories_by_name[repository['name']] = repository
            if debug:
                prettyllog("semaphore", "get", repository['name'], "ok", response.status_code , "loadning repositories", severity="DEBUG")
        prettyllog("semaphore", "get", "repository", "ok", response.status_code , "loadning repositories", severity="INFO")
        return repositories_by_name
    else:
        # Failed request
        prettyllog("semaphore", "get", "repository", "error", response.status_code , "loadning repositories", severity="ERROR")

    
def create_project(session, project):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    project_url = f"{baseurl}/api/projects"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.post(project_url, headers=headers, json=project)
    if response.status_code == 201:
        # Successful request
        prettyllog("semaphore", "create", project['name'], "ok", response.status_code , "create project", severity="INFO")
        return response.json()
    else:
        # Failed request
        prettyllog("semaphore", "create", project['name'], "error", response.status_code , "create project", severity="ERROR")


def get_project(session):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    project_url = f"{baseurl}/api/projects"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.get(project_url, headers=headers)
    if response.status_code == 200:
        # Successful request
        projects = response.json()
        # map prokects by name
        projects_by_name = {}
        for project in projects:
            projects_by_name[project['name']] = project
            if debug:
                prettyllog("semaphore", "get", project['name'], "ok", response.status_code , "loadning projects", severity="DEBUG")
        prettyllog("semaphore", "get", "project", "ok", response.status_code , "loadning projects", severity="INFO")
        return projects_by_name
    else:
        # Failed request
        prettyllog("semaphore", "get", "project", "error", response.status_code , "loadning projects", severity="ERROR")

def update_inventory(session, project_id, inventory_id, inventory):
    prettyllog("semaphore", "update", inventory['name'], "ok", 0 , "update inventory", severity="INFO")
    # check if inventory exists
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    inventory_url = f"{baseurl}/api/project/{project_id}/inventory/{inventory_id}"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }
    inventory['id'] = inventory_id
    response = session.put(inventory_url, headers=headers, json=inventory)
    if response.status_code == 204:
        # Successful request
        prettyllog("semaphore", "update", inventory['name'], "ok", response.status_code , "update inventory", severity="INFO")
        return response.json()
    else:
        # Failed request
        prettyllog("semaphore", "update", inventory['name'], "error", response.status_code , "update inventory", severity="ERROR")

def create_inventory(session, project_id, inventory):
    prettyllog("semaphore", "create", inventory['name'], "ok", 0 , "create inventory", severity="INFO")
    # check if inventory exists

    myinv =  get_inventory(session, project_id, inventory['name'])
    try:
        if myinv[inventory['name']]:
            prettyllog("semaphore", "update", inventory['name'], "ok", 0 , "update inventory", severity="INFO")
            myinvid = myinv[inventory['name']]['id']
            update_inventory(session, project_id, myinvid, inventory)
            return True
    except:
        baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
        inventory_url = f"{baseurl}/api/project/{project_id}/inventory"  # Adjust the URL as needed
        headers = {
            'accept': 'application/json',
            'Content-Type': 'application/json'
        }
    # Use the session for the request
        response = session.post(inventory_url, headers=headers, json=inventory)
    
        if response.status_code == 201:
        # Successful request
            prettyllog("semaphore", "create", inventory['name'], "ok", response.status_code , "create inventory", severity="INFO")
            return response.json()
        else:
            # Failed request
            prettyllog("semaphore", "create", inventory['name'], "error", response.status_code , "create inventory", severity="ERROR")

def create_credemtial(session, project_id, credential):
    prettyllog("semaphore", "create", credential['name'], "ok", 0 , "create credential", severity="INFO")


    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    credential_url = f"{baseurl}/api/project/{project_id}/keys"  # Adjust the URL as needed
    
    pprint.pprint(credential_url)

    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }
    # Use the session for the request
    response = session.get(credential_url, headers=headers)
    if response.status_code == 200:
        known_keys = response.json()
        for key in known_keys:
            if key['name'] == credential['name']:
                prettyllog("semaphore", "create", credential['name'], "ok", response.status_code , "create credential", severity="INFO")
                return True
    else:
        # Failed request
        prettyllog("semaphore", "create", credential['name'], "error", response.status_code , "create credential", severity="DEBUG")

    response = session.post(credential_url, headers=headers, json=credential)
    pprint.pprint(response.content)
    if response.status_code == 204:
        # Successful request
        prettyllog("semaphore", "create", credential['name'], "ok", response.status_code , "create credential", severity="INFO")
        return True
    else:
        if response.status_code == 404:
            # Failed request
            prettyllog("semaphore", "create", credential['name'], "error", response.status_code , "create credential", severity="ERROR")
            pprint.pprint(response.content)
            return False

        # Failed request
        prettyllog("semaphore", "create", credential['name'], "error", response.status_code , "create credential", severity="ERROR")

def popoulate_inventory(session, project_id, inventory_id, inventory):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    inventory_url = f"{baseurl}/api/project/{project_id}/inventory/{inventory_id}"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }
    
    response = session.get(inventory_url, headers=headers)

    if response.status_code == 201:
        # Successful request
        prettyllog("semaphore", "create", inventory['name'], "ok", response.status_code , "create inventory", severity="INFO")
    else:
        # Failed request
        prettyllog("semaphore", "create", inventory['name'], "error", response.status_code , "create inventory", severity="ERROR")
        return False
    
    inventorydata = response.json()
    inventorydata[''] = inventory


def get_inventory(session, project_id, projectname):
    prettyllog("semaphore", "get", "inventory", "ok", 0 , "loadning inventory %s" % projectname, severity="INFO")
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    inventory_url = f"{baseurl}/api/project/{project_id}/inventory?sort=name&order=asc' "  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }
    response = session.get(inventory_url, headers=headers)
    if response.status_code == 200:
        # Successful request
        inventory = response.json()
        # map projects by name
        inventory_by_name = {}
        for item in inventory:
            prettyllog("semaphore", "get", item['name'], "ok", response.status_code , "loadning inventory", severity="INFO")
            inventory_by_name[item['name']] = item
            if debug:
                prettyllog("semaphore", "get", item['name'], "ok", response.status_code , "loadning inventory", severity="DEBUG")
        prettyllog("semaphore", "get", "inventory", "ok", response.status_code , "loadning inventory", severity="INFO")
        return inventory_by_name
    else:
        # Failed request
        prettyllog("semaphore", "get", "inventory", "error", response.status_code , "loadning inventory", severity="ERROR")

def check_env():
    envok = True
    myenv = {}
    known_git_types = ['github', 'gitlab', 'bitbucket', 'gitea', 'gogs']
    if (os.getenv('MANISBLE_SEMAPHORE_URL') == None):
        prettyllog("semaphore", "Init", "env", "error", 1 , "MANISBLE_SEMAPHORE_URL not set", severity="ERROR")
        envok = False
    else:
        myenv['MANISBLE_SEMAPHORE_URL'] = os.getenv('MANISBLE_SEMAPHORE_URL')
        prettyllog("semaphore", "Init", "env", "ok", 0 , "MANISBLE_SEMAPHORE_URL set", severity="INFO")

    if (os.getenv('MANISBLE_SEMAPHORE_USER') == None):
        prettyllog("semaphore", "Init", "env", "error", 1 , "MANISBLE_SEMAPHORE_USER not set", severity="ERROR")
        envok = False
    else:       
        myenv['MANISBLE_SEMAPHORE_USER'] = os.getenv('MANISBLE_SEMAPHORE_USER')
        prettyllog("semaphore", "Init", "env", "ok", 0 , "MANISBLE_SEMAPHORE_USER set", severity="INFO")

    if (os.getenv('MANISBLE_SEMAPHORE_PASSWORD') == None):
        prettyllog("semaphore", "Init", "env", "error", 1 , "MANISBLE_SEMAPHORE_PASSWORD not set", severity="ERROR")
        envok = False
    else:
        myenv['MANISBLE_SEMAPHORE_PASSWORD'] = os.getenv('MANISBLE_SEMAPHORE_PASSWORD')
        prettyllog("semaphore", "Init", "env", "ok", 0 , "MANISBLE_SEMAPHORE_PASSWORD set", severity="INFO")

    if (os.getenv('MANISBLE_GIT_TYPE') == None):
        prettyllog("semaphore", "Init", "env", "error", 1 , "MANISBLE_GIT_TYPE not set", severity="ERROR")
        envok = False
    elif (os.getenv('MANISBLE_GIT_TYPE') not in known_git_types):
        prettyllog("semaphore", "Init", "env", "error", 1 , "MANISBLE_GIT_TYPE not set %s " % known_git_types, severity="ERROR")
        envok = False
    else:
        myenv['MANISBLE_GIT_TYPE'] = os.getenv('MANISBLE_GIT_TYPE')
        prettyllog("semaphore", "Init", "env", "ok", 0 , "MANISBLE_GIT_TYPE set", severity="INFO")
    return envok, myenv



def read_config():
    f = open("etc/manisble/manisble.json", "r")
    mainconf = json.load(f)
    prettyllog("semaphore", "Init", "main","main configuration", 0 , "Getting main config", severity="INFO")
    f.close()
    subconf = {}

    # check if automation is enabled for semaphore
    for subproject in mainconf['subprojects']:
        if subproject['engine'] == 'semaphore':
            subconf[subproject['name']] = subproject
            prettyllog("semaphore", "Init", "subpropject", subproject['name'] , "000", "Getting subproject config", severity="DEBUG")
    return True, mainconf, subconf



    



        


 # Example: Get a list of your projects

def check_project(projectname, env):
    # we need to check if the project exists in git
    # if not we need to create it
    # if it exists we need to check if it exists in semaphore
    # if not we need to create it
    if env['MANISBLE_GIT_TYPE'] == 'gitea':
        from ..gitea.git import get_git_projects
        git_projects = get_git_projects()
        git_project_names = []
        for git_project in git_projects:
            git_project_names.append(git_project['name'])
        if projectname in git_project_names:
            prettyllog("semaphore", "Check", projectname, "ok", 0 , "project exists in git", severity="INFO")
            # check if project exists in semaphore
            # if not create it
            # if it exists check if it has the correct settings
            # if not update it
        else:
            prettyllog("semaphore", "Create", projectname, "missing", 1 , "project missing in git", severity="WARNING")
            from ..gitea.git import create_git_project 
            project = {}
            project['name'] = projectname
            project['description'] = "manisble project"
            project['private'] = True
            project['auto_init'] = True

            # create project in git
            create_git_project(project)
            # create a temporary dir and clone the project
        prettyllog("semaphore", "Init", "clone", projectname , "000", "manisble.json missing", severity="DEBUG")
        configdata = clone_git_project(projectname)
        pprint.pprint(configdata)
            # create project in semaphore

            # create project in git
            # create project in semaphore
            # create project in awx
            # create project in vault
            # create project in dns
    else:
        prettyllog("project", "semaphore", projectname, "error", 1 , "git type not supported", severity="ERROR")
        exit(1)



    



    


def get_sshkey_id(session, project_id, sshkeyname):
    baseurl = os.getenv('MANISBLE_SEMAPHORE_URL')
    sshkey_url = f"{baseurl}/api/project/{project_id}/keys?name=%s" % sshkeyname  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }
    response = session.get(sshkey_url, headers=headers)
    if response.status_code == 200:
        # Successful request
        sshkey = response.json()
        # map keys by name
        try:
            sshkeyid = sshkey[0]['id']
        except:
            sshkeyid = None
        return sshkeyid
    else:
        # Failed request
        prettyllog("semaphore", "get", "sshkey", "error", response.status_code , "loadning sshkeys", severity="ERROR")

def main():

    semaphore, mainconf, subprojects = read_config()
    organization = mainconf['organisation']['name']
    mysubprojects = mainconf['subprojects']
    checkenv, env = check_env()
    if not checkenv:
        prettyllog("semaphore", "main", "config", "error", 1 , "No semaphore config found", severity="ERROR")
        exit(1)

    state = {}
    session = login()
    # Check if the main project exists , it is the organization name
    # if not create it
    # if it exists check if it has the correct settings
    # if not update it
    prettyllog("semaphore", "main", "config", organization, 1 , "Check the main project", severity="INFO")
    check_project(organization, env)




    if session:
        projects = get_project(session)
        for project in projects:
            projectname = projects[project]['name']
            state[projectname] = {}
            state[projectname]['project'] = projects[project]
            state[projectname]['inventory'] = {}
            inventory = get_inventory(session, projects[project]['id'], projectname)
            for item in inventory:
                itemname = inventory[item]['name']
                state[projectname]['inventory'][itemname] = {}
                state[projectname]['inventory'][itemname]['item'] = inventory[item]
                prettyllog("semaphore", "main", item, "ok", 0 , "item", severity="INFO")

    missings = {}
    for subproject in subprojects:
        if subproject in state:
            prettyllog("semaphore", "main", subproject, "ok", 0 , "subproject", severity="INFO")
        else:
            prettyllog("semaphore", "main", subproject, "missing", 1 , "subproject", severity="DEBUG")
            missings[subproject] = subprojects[subproject]

    orphans = {}    
    for state_project in state:
        if state_project in subprojects or state_project == organization:
            prettyllog("semaphore", "check project", state_project, "ok", 0 , "subproject", severity="INFO")
        else:
            prettyllog("semaphore", "check project", state_project, "orphan", 1 , "subproject", severity="WARNING")
            orphans[state_project] = state[state_project]


    for missing in missings:
        prettyllog("semaphore", "createi project", missing, "Preparing to create project", 000 , "subproject", severity="DEBUG")
        # create project
        check_project(missing, env)
        create_project(session, missings[missing])

    for orphan in orphans:
        prettyllog("semaphore", "delete proejct", orphan, "Preparing to delete project", 000 , "subproject", severity="DEBUG")
        # delete project
        # delete project in git
        # delete project in semaphore
        # delete project in awx
        # delete project in vault
        # delete project in dns

    # prettyllog("semaphore", "main", "main", "ok", 0 , "subproject", severity="INFO")

    projects = get_project(session)
    for project in projects:
        prettyllog("semaphore", "projects", project, "ok", 0 , "project", severity="INFO")
        # We need to check if the project exists in git
    
        projectname = projects[project]['name']
        state[projectname] = {}
        state[projectname]['project'] = projects[project]
        state[projectname]['inventory'] = {}
        sshkey  = {
            "name": "dummy",
            "type": "none",
            "project_id": projects[project]['id']
        }
        create_credemtial(session, projects[project]['id'] , sshkey)

        becomekey  = {
            "name": "dummyroot",
            "type": "none",
            "project_id": projects[project]['id']
        }
        create_credemtial(session, projects[project]['id'] , becomekey)

        ssh_key_id = get_sshkey_id(session, projects[project]['id'], sshkey['name'])
        become_key_id = get_sshkey_id(session, projects[project]['id'], becomekey['name'])

        myinventory = get_netbox_inventory_from_tag(projectname)
        myinvdata = get_inventory(session, projects[project]['id'], projectname)
        invexists = False
        try:
            if myinvdata[projectname]:
               invexists = True 
        except:
            invexists = False
        pprint.pprint(invexists)
        if not invexists:
            invetoryname = "%s-%s" % (projectname, "netbox")
            inventorydata = {
                        "name": invetoryname,
                        "project_id": projects[project]['id'],
                        "inventory": myinventory,
                        "ssh_key_id": ssh_key_id,
                        "become_key_id": become_key_id,
                        "type": "static"
            }
            create_inventory(session, projects[project]['id'], inventorydata)
    return 0







import requests
import os
from pyVmomi import vim
import atexit
import json
import pprint
import pickle
import redis
import time
from pyVim.connect import SmartConnect, Disconnect
from ..common import prettyllog

print("Starting vmware")    
print("-------------------------------------------------------")
redishost=os.getenv("MANISBLE_REDIS_HOST")
if redishost is None:
    redishost="localhost"

    
redisport=os.getenv("MANISBLE_REDIS_PORT")
if redisport is None:
    redisport="6379"

redisdb=os.getenv("MANISBLE_REDIS_DB")
if redisdb is None:
    redisdb="0"
print("Using redis host: %s" % redishost)
print("Using redis port: %s" % redisport)
print("Using redis db: %s" % redisdb)

print("-------------------------------------------------------")

r = redis.Redis(host=redishost, port=redisport, db=redisdb)

data = {}


def connect(env):
    try:
        if env['MANISBLE_VMWARE_SSL'] == "False":
            prettyllog("vsphere", "init", "Connect to vcenter", "start", "000" , "Connecting without ssl verify", severity="INFO")
            try:
                service_instance = SmartConnect(host=env['MANISBLE_VMWARE_HOST'],
                                            user=env['MANISBLE_VMWARE_USERNAME'],
                                            pwd=env['MANISBLE_VMWARE_PASSWORD'],
                                            disableSslCertValidation=True)
            except IOError as io_error: 
                print(io_error)
                raise SystemExit("Unable to connect to host with supplied credentials.")
        else:
            prettyllog("vsphere", "init", "Connect to vcenter", "start", "000" , "Connecting with ssl verify", severity="INFO")
            try:
                service_instance = SmartConnect(host=myhost,
                                            user=env['MANISBLE_VMWARE_USERNAME'],
                                            pwd=env['MANISBLE_VMWARE_PASSWORD'])
            except: 
                # this is fatal
                print("fatal")

                
        #atexit.register(Disconnect, service_instance)
    except IOError as io_error:
        print(io_error)
    if not service_instance:
        raise SystemExit("Unable to connect to host with supplied credentials.")
    return service_instance


def  get_env():

  myenv = {}
  try:
    myenv['MANISBLE_VMWARE_HOST'] = os.getenv("MANISBLE_VMWARE_HOST")
    myenv['MANISBLE_VMWARE_USERNAME'] = os.getenv("MANISBLE_VMWARE_USERNAME")
    myenv['MANISBLE_VMWARE_PASSWORD'] = os.getenv("MANISBLE_VMWARE_PASSWORD")
    myenv['MANISBLE_VMWARE_SSL'] = os.getenv("MANISBLE_VMWARE_SSL")
  except KeyError as key_error:
    print(key_error)
    raise SystemExit("Unable to get environment variables.")
  
  
  return myenv


# Function to get a list of VMs

def get_cluster_list(content):
    limit = 20
    cluster_list = []
    cluster_details = {}
    container = content.viewManager.CreateContainerView(
        content.rootFolder, [vim.ClusterComputeResource], True
    )
    # promnt items in container
    pprint.pprint(container.view)

    for cluster in container.view:
        if limit == 0:
            break
        limit -= 1
        cluster_list.append(cluster.name)
        cluster_details[cluster.name] = {}
        cluster_details[cluster.name]['configuration'] = cluster.configuration
        cluster_details[cluster.name]['datastore'] = cluster.datastore
        cluster_details[cluster.name]['host'] = cluster.host
        cluster_details[cluster.name]['network'] = cluster.network
        cluster_details[cluster.name]['summary'] = cluster.summary
    return cluster_list, cluster_details

def get_vm_details(content, vmname):
    vm_details = {}
    container = content.viewManager.CreateContainerView(
        content.rootFolder, [vim.VirtualMachine], True
    )
    for vm in container.view:
        if vm.name == vmname:
            vm_details[vm.name] = {}
            vm_details[vm.name]['config'] = vm.summary.config
            vm_details[vm.name]['guest'] = vm.summary.guest
            vm_details[vm.name]['runtime'] = vm.summary.runtime
            vm_details[vm.name]['storage'] = vm.summary.storage
            vm_details[vm.name]['summary'] = vm.summary
            return vm_details

def get_vm_list(content):
    vm_list = []
    vm_details = {}
    container = content.viewManager.CreateContainerView(
        content.rootFolder, [vim.VirtualMachine], True
    )
    for vm in container.view:
        vm_list.append(vm.name)
        vm_details[vm.name] = {}
        #vm_details[vm.name]['config'] = vm.summary.config
        #vm_details[vm.name]['guest'] = vm.summary.guest
        #vm_details[vm.name]['runtime'] = vm.summary.runtime
        #vm_details[vm.name]['storage'] = vm.summary.storage
        #vm_details[vm.name]['summary'] = vm.summary
    return vm_list, vm_details

def vm2dict(datacenter, cluster, host, vm, summary):
    # If nested folder path is required, split into a separate function
    vmname = vm.summary.config.name
    data[datacenter][cluster][host][vmname]['folder'] = vm.parent.name
    data[datacenter][cluster][host][vmname]['mem'] = summary['mem']
    data[datacenter][cluster][host][vmname]['diskGB'] = summary['diskGB']
    data[datacenter][cluster][host][vmname]['cpu'] = summary['cpu']
    data[datacenter][cluster][host][vmname]['path'] = summary['path']
    data[datacenter][cluster][host][vmname]['net'] = summary['net']
    data[datacenter][cluster][host][vmname]['ostype'] = summary['ostype']
    data[datacenter][cluster][host][vmname]['state'] = summary['state']
    data[datacenter][cluster][host][vmname]['annotation'] = summary['annotation']


def data2json(raw_data, args):
    with open(args.jsonfile, 'w') as json_file:
        json.dump(raw_data, json_file)    

def get_nics(guest):
    nics = {}
    for nic in guest.net:
        if nic.network:  # Only return adapter backed interfaces
            if nic.ipConfig is not None and nic.ipConfig.ipAddress is not None:
                nics[nic.macAddress] = {}  # Use mac as uniq ID for nic
                nics[nic.macAddress]['netlabel'] = nic.network
                ipconf = nic.ipConfig.ipAddress
                i = 0
                nics[nic.macAddress]['ipv4'] = {}
                for ip in ipconf:
                    if ":" not in ip.ipAddress:  # Only grab ipv4 addresses
                        nics[nic.macAddress]['ipv4'][i] = ip.ipAddress
                        nics[nic.macAddress]['prefix'] = ip.prefixLength
                        nics[nic.macAddress]['connected'] = nic.connected
                    i = i+1
    return nics


def vmsummary(summary, guest):
    vmsum = {}
    config = summary.config
    net = get_nics(guest)
    vmsum['mem'] = str(config.memorySizeMB / 1024)
    vmsum['diskGB'] = str("%.2f" % (summary.storage.committed / 1024**3))
    vmsum['cpu'] = str(config.numCpu)
    vmsum['path'] = config.vmPathName
    vmsum['ostype'] = config.guestFullName
    vmsum['state'] = summary.runtime.powerState
    vmsum['annotation'] = config.annotation if config.annotation else ''
    vmsum['net'] = net

    return vmsum



# Connect to the vCenter server
def connect_to_vcenter(debug=True):
    prettyllog("vsphere", "init", "Connect to vcenter", "start", "000" , "Connecting", severity="INFO")
    myenv = get_env()
    service_instance = connect(myenv)
    if debug:
        print(parse_service_instance(service_instance))
    if not service_instance:
        prettyllog("vsphere", "init", "Connect to vcenter", "error", "000" , "Fail to Connect", severity="ERROR")
        return None
    prettyllog("vsphere", "init", "Connect to vcenter", "ok", "000" , "Connected", severity="INFO")
    return service_instance.RetrieveContent()


def get_obj(content, vimtype, name = None):
    return [item for item in content.viewManager.CreateContainerView(
        content.rootFolder, [vimtype], recursive=True).view]


def parse_service_instance(si):
    """
    Print some basic knowledge about your environment as a Hello World
    equivalent for pyVmomi
    """

    content = si.RetrieveContent()
    pprint.pprint(content.about)
    print("")
    print("Content:")
    print(" - content.about                 = %s" % content.about)
    print(" - content.about.apiType         = %s" % content.about.apiType)
    print(" - content.about.instanceUuid    = %s" % content.about.instanceUuid)
    print(" - content.about.osType          = %s" % content.about.osType)
    print(" - content.about.productLineId   = %s" % content.about.productLineId)
    
    object_view = content.viewManager.CreateContainerView(content.rootFolder,
                                                          [], True)
    for obj in object_view.view:
        print(obj)
    object_view.Destroy()
    return 0



# MAIN
# We are serving vspere and we need an infinite loop
prettyllog("vsphere", "init", "service", "ok", "000" , "starting server", severity="INFO")
myenv = get_env()
for key, value in myenv.items():
    if value is None:
        prettyllog("vsphere", "init", "service", "error", "000" , "environment variable %s not set" % key, severity="ERROR")
        raise SystemExit("Unable to get environment variables.")
    else:
        prettyllog("vsphere", "init", "service", "ok", "000" , "environment variable %s = %s" % (key, value), severity="INFO")




orphans = []
while True:
    content = connect_to_vcenter()
    vm_list, vm_details = get_vm_list(content)
    totalservers = len(vm_list)
    for vm in vm_list:
        knownservers = r.keys("manisble:vmware:*:known")
        detailedservers = r.keys("manisble:vmware:*:details")

        knownservercount = len(knownservers)
        detailedservercount = len(detailedservers)
        prettyllog("vsphere", "get", vm, "check", "000" , "loadning servers (totalservers: %s known: %s updated %s )" % (totalservers, knownservercount, detailedservercount), severity="INFO")
        detailkey = "manisble:vmware:" + vm + ":details"
        knownkey = "manisble:vmware:" + vm + ":known"
        knownlinux = "manisble:vmware:" + vm + ":known:linux"
        knownwindows = "manisble:vmware:" + vm + ":known:windows"
        timestamp = time.time()
        get = r.get(detailkey)
        if get is None:
            prettyllog("vsphere", "get", vm, "new", "000" , "loadning servers", severity="CHANGE")
            vm_details = get_vm_details(content, vm)
            memory = vm_details[vm]['config'].memorySizeMB
            numCpu = vm_details[vm]['config'].numCpu
            numEthernetCards = vm_details[vm]['config'].numEthernetCards
            numVirtualDisks = vm_details[vm]['config'].numVirtualDisks
            uuid = vm_details[vm]['config'].uuid
            guestId = vm_details[vm]['config'].guestId
            guestFullName = vm_details[vm]['config'].guestFullName
            toolsStatus = vm_details[vm]['guest'].toolsStatus
            toolsVersionStatus = vm_details[vm]['guest'].toolsVersionStatus
            toolsVersionStatus2 = vm_details[vm]['guest'].toolsVersionStatus2
            toolsRunningStatus = vm_details[vm]['guest'].toolsRunningStatus
            hostName = vm_details[vm]['guest'].hostName
            ipAddress = vm_details[vm]['guest'].ipAddress
            hwVersion = vm_details[vm]['config'].hwVersion
            vmPathName = vm_details[vm]['config'].vmPathName
            overallStatus = vm_details[vm]['summary'].overallStatus
            mydetails = {}
            mydetails['memory'] = memory
            mydetails['numCpu'] = numCpu
            mydetails['numEthernetCards'] = numEthernetCards
            mydetails['numVirtualDisks'] = numVirtualDisks
            mydetails['uuid'] = uuid
            mydetails['guestId'] = guestId
            mydetails['guestFullName'] = guestFullName
            mydetails['toolsStatus'] = toolsStatus
            mydetails['toolsVersionStatus'] = toolsVersionStatus
            mydetails['toolsVersionStatus2'] = toolsVersionStatus2
            mydetails['toolsRunningStatus'] = toolsRunningStatus
            mydetails['hostName'] = hostName
            mydetails['ipAddress'] = ipAddress
            mydetails['hwVersion'] = hwVersion
            mydetails['vmPathName'] = vmPathName
            mydetails['overallStatus'] = overallStatus
            myjson = json.dumps(mydetails)
            r.set(detailkey, myjson, ex=3600000)
            r.set(knownkey, str(timestamp))
            if "Linux" in guestFullName or "CentOS" in guestFullName or "Ubuntu" in guestFullName or "Debian" in guestFullName or "Red Hat" in guestFullName or "SUSE" in guestFullName or "Fedora" in guestFullName or "Oracle" in guestFullName or "CoreOS" in guestFullName or "Photon" in guestFullName or "VMware" in guestFullName or "FreeBSD" in guestFullName:
                r.set(knownlinux, str(timestamp))
                prettyllog("vsphere", "get", vm, guestFullName, "000" , "Linux detected", severity="CHANGE")

            if "Windows" in guestFullName:
                r.set(knownwindows, str(timestamp))
                prettyllog("vsphere", "get", vm, guestFullName, "000" , "Windows detected", severity="CHANGE")

            prettyllog("vsphere", "get", vm, guestFullName, "000" , "loadning servers", severity="CHANGE")
        else:
            prettyllog("vsphere", "get", vm, get, "000" , "loadning servers", severity="INFO")
            r.set(knownkey, str(timestamp))







#       hosts = cluster.host  # Variable to make pep8 compliance
#        for host in hosts:  # Iterate through Hosts in the Cluster
#            hostname = host.summary.config.name
#            # Add VMs to data dict by config name
#            data[datacenter.name][cluster.name][hostname] = {}
#            vms = host.vm
#            for vm in vms:  # Iterate through each VM on the host
#                vmname = vm.summary.config.name
#                data[datacenter.name][cluster.name][hostname][vmname] = {}
#                summary = vmsummary(vm.summary, vm.guest)
#                vm2dict(datacenter.name, cluster.name, hostname, vm, summary)





  


import requests
import json
import os
import base64
import xml.etree.ElementTree as ET
import platform
import yaml
from ..common import prettyllog
import pprint


ZABBIX_API_URL = os.environ.get('ZABBIX_URL') + "/api_jsonrpc.php"
AUTHTOKEN = os.environ.get('ZABBIX_TOKEN')

#ZABBIX_API_URL = "https://zabbix.openknowit.com/api_jsonrpc.php"
#AUTHTOKEN = "9e39c4605ff25083f230b22ccaf1c18128fb8123a502abddeaad73e0b6cff0b8"
#  --data '{"jsonrpc":"2.0","method":"item.create","params":{"name":"Free disk space on /home/joe/","key_":"vfs.fs.size[/home/joe/,free]","hostid":"10084","type":0,"value_type":3,"interfaceid":"1","delay":30},"id":3}'

def list_host_group():
    HOSTGROUP= os.environ.get('ZABBIX_HOSTGROUP')
    r = requests.post(ZABBIX_API_URL,
    json= {     
          "jsonrpc": "2.0",     
          "method": "hostgroup.get",     
          "params": {         
          "output": "extend",
                "filter": {
                "name": [
                    HOSTGROUP
                        ]         
                }

          },         
        "auth": AUTHTOKEN
    })
    print(json.dumps(r.json(), indent=4, sort_keys=True))

def list_host_groups():
    prettyllog("List host groups","info",   "zabbix", "list_host_groups", "zabbix.py", "manisble")
    r = requests.post(ZABBIX_API_URL,
    json= {     
          "jsonrpc": "2.0",     
          "method": "hostgroup.get",     
          "params": {         
          "output": "extend",
                 "filter": {
                 "name": [
            ] 
            }
        },
        "id": 1,
        "auth": AUTHTOKEN
    })
    pprint.pprint(r.content)
    
    print(json.dumps(r.json(), indent=4, sort_keys=True))

def get_host_id(hostname):
    r = requests.post(ZABBIX_API_URL,
    json= {

            "jsonrpc": "2.0",
            "method": "host.get",
            "params": {
                "output": "extend",
                "filter": {
                    "host": [
                        hostname
                    ]
                }
            },
            "auth": AUTHTOKEN,
            "id": 1
        })
    try:
        return r.json()['result'][0]['hostid']
    except:
        print("no host found")
        return None
    
def get_host_group_id(hostgroup):
    r = requests.post(ZABBIX_API_URL,
    json= {

            "jsonrpc": "2.0",
            "method": "hostgroup.get",
            "params": {
                "output": "extend",
                "filter": {
                    "name": [
                        hostgroup
                    ]
                }
            },
            "auth": AUTHTOKEN,
            "id": 1
        })
    try:
        return r.json()['result'][0]['groupid']
    except:
        print("no hostgroup found")
        return None
    
def create_host(hostname , hostgroup_id, ipadress):    
    r = requests.post(ZABBIX_API_URL,
    json= {     
          "jsonrpc": "2.0",     
          "method": "host.create",     
          "params": {         
          "host": hostname,
          "interfaces": [
            {
                "type": 1,
                "main": 1,
                "useip": 1,
                "ip": ipadress,
                "dns": "",
                "port": "10050"
            }
        ],
        "groups": [
            {
                "groupid": hostgroup_id
            }
        ],
        "tags": [
            {
                "tag": "Host name",
                "value": hostname
            }
        ],
        "inventory_mode": 0,
        "inventory": {
            "macaddress_a": "01234",
            "macaddress_b": "56768"
        }
          },
        "id": 1,
        "auth": AUTHTOKEN
    })
    try:
        return r.json()['result']['hostids'][0]
    except:
        print(r.status_code)
        print(r.content)
        print("no host created")
        return None
    
def  host_update(hostname, host_id, hostgroup_id, ipadress):
    r = requests.post(ZABBIX_API_URL,
    json= {     
          "jsonrpc": "2.0",     
          "method": "host.update",     
          "params": {         
          "hostid": host_id,
          "interfaces": [
            {
                "type": 1,
                "main": 1,
                "useip": 1,
                "ip": ipadress,
                "dns": "",
                "port": "10050"
            }
        ],
        "groups": [
            {
                "groupid": hostgroup_id
            }
        ],
        "tags": [
            {
                "tag": "Host name",
                "value": hostname
            }
        ],
        "inventory_mode": 0,
        "inventory": {
            "macaddress_a": "01234",
            "macaddress_b": "56768"
        }
          },
        "id": 1,
        "auth": AUTHTOKEN
    })
    try:
        return r.json()['result']['hostids'][0]
    except:
        print(r.status_code)
        print(r.content)
        print("host not updated")
        return None

def create_host_group(hostgroup = "Linux servers"):
    r = requests.post(ZABBIX_API_URL,
    json= {     
          "jsonrpc": "2.0",     
          "method": "hostgroup.create",     
          "params": {         
          "name": hostgroup
          },     
        "auth": AUTHTOKEN
    })
    try:
        return r.json()['result']['groupids'][0]
    except:
        print("no hostgroup created")
        return None
    
def status():
    print("status")
    return 0


def register():
    print("register")
    try:
        hostname = os.environ.get('MANISBLE_ZABBIX_HOSTNAME')
    except:
        pass
    if hostname == None:
        print("no hostname defined in env MANISBLE_ZABBIX_HOSTNAME")
        return 1
    print("hostname: " + hostname)
    try:
        ipadress = os.environ.get('MANISBLE_ZABBIX_HOSTIP')
    except:
        pass
    if ipadress == None:
        print("no ipadress defined in env MANISBLE_ZABBIX_HOSTIP")
        return 1
    print("ipadress: " + ipadress)


    try:
        hostgroup = os.environ.get('MANISBLE_ZABBIX_HOSTGROUP')
    except:
        pass
    if hostgroup == None:
        print("no hostgroup defined in env MANISBLE_ZABBIX_HOSTGROUP")
        return 1
    
    print("hostgroup: " + hostgroup)
    hostgroup_id = get_host_group_id(hostgroup)
    if hostgroup_id == None:
        print("no hostgroup found")
        hostgroup_id = create_host_group(hostgroup)
        if hostgroup_id == None:
            print("no hostgroup created")
            return 1
        else:
            print("hostgroup created")
    else:
        print("hostgroup found")

    print("hostgroup_id: " + hostgroup_id)
    host_id = get_host_id(hostname)
    if host_id == None:
        print("no host found")
        host_id = create_host(hostname, hostgroup_id, ipadress)
        if host_id == None:
            print("no host created")
            return 1
        else:
            print("host created")
    else:
        print("host found")
        host_id = host_update(hostname, host_id, hostgroup_id, ipadress)
        if host_id == None:
            print("no host updated")
            return 1
        else:
            print("host updated")
    print("host_id: " + host_id)
    hostdate = get_host_data(host_id)
    open("hostdata.yaml", "w").write(yaml.dump(hostdate))
    return 0

def get_host_data(host_id):
    r = requests.post(ZABBIX_API_URL,
    json= {

            "jsonrpc": "2.0",
            "method": "host.get",
            "params": {
                "output": "extend",
                "filter": {
                    "hostid": [
                        host_id
                    ]
                }
            },
            "auth": AUTHTOKEN,
            "id": 1
        })
    try:
        return r.json()['result'][0]
    except:
        print("no host found")
        return None
    



import requests
import os
from pyVmomi import vim
import atexit
import json
import pprint
import redis
import time
from pyVim.connect import SmartConnect, Disconnect
from ..common import prettyllog
from .common import get_env
from .organization import refresh_netbox_orgs
from .clusters import get_cluster_id, get_clusters
from .redis import refresh_netbox_from_redis
from .subprojects import update_subprojects



r = redis.Redis(host='localhost', port=6379, db=0)
data = {}

# MAIN
# We are serving vspere and we need an infinite loop
def main():
    prettyllog("netbox", "init", "service", "manisble" "ok", "000" , "starting server", severity="INFO")
    myenv = get_env()
    while True:
        netboxdata = refresh_netbox_orgs(myenv)
        prettyllog("netbox", "init", "service", "redis" "ok", "000" , "updating cached servers", severity="INFO")
        refresh_netbox_from_redis(myenv, netboxdata)
        update_subprojects(myenv)
        time.sleep(60)

# Refresh netbox from redis
#       clusters=get_clusters(myenv)
#       for cluster in clusters:
#           cluster_id = get_cluster_id(cluster, myenv)
#           if cluster_id:
#               prettyllog("netbox", "get", "cluster", cluster, cluster_id , "getting cluster id", severity="INFO")
#           else:
#               prettyllog("netbox", "get", "cluster", cluster, "000" , "unable to get cluster", severity="ERROR")
#       break








  


#!/usr/bin/env python
# VMware vSphere Python SDK
# Copyright (c) 2008-2021 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Python program to authenticate and print
a friendly encouragement to joining the community!
"""

from pyVmomi import vmodl
from tools import cli, service_instance


def main():
    """
    Simple command-line program for listing the virtual machines on a system.
    """

    parser = cli.Parser()
    args = parser.get_args()
    si = service_instance.connect(args)

    try:
        print("\nHello World!\n")
        print("If you got here, you authenticted into vCenter.")
        print("The server is {}!".format(args.host))
        # NOTE (hartsock): only a successfully authenticated session has a
        # session key aka session id.
        session_id = si.content.sessionManager.currentSession.key
        print("current session id: {}".format(session_id))
        print("Well done!")
        print("\n")
        print("Download, learn and contribute back:")
        print("https://github.com/vmware/pyvmomi-community-samples")
        print("\n\n")

    except vmodl.MethodFault as error:
        print("Caught vmodl fault : " + error.msg)
        return -1

    return 0


# Start program
if __name__ == "__main__":
    main()



from . import pitv
import argparse
from ..common import prettyllog




def main():
    parser = argparse.ArgumentParser(description="Keep manisble and automate", usage="manisble_pitv <action> \n\n \
               \
               version : 0.1.2 pitv  \n                                              \
               actions:\n                                                      \
               status        status pitv \n  \
               \
               2023 Knowit Miracle\
               ")
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup jenkis')
    args = parser.parse_args()
    ready = False
    print("check if we are ready to go")


    if args.action[0] == "evacuate":
        pitv.evacuate()

    if args.action[0] == "status":
        print("status pitv")
        pitv.status()


    if args.action[0] == "list_datasets":
        print("list_datasets pitv")
        pitv.list_datasets()
    
    if args.action[0] == "delete_all_datasets":
        print("delete_all_datasets pitv")
        pitv.delete_all_datasets()
        

    if args.action[0] == "delete_all_dags":
        print("delete_all_dags pitv")
        pitv.delete_all_dags()


    if args.action[0] == "set":
        print("set pitv")
        pitv.set()

    if args.action[0] == "list_dags":
        print("list_dags pitv")
        pitv.list_dags()






import os
import json


def usage():
  # export the environment variables
  print("export MANISBLE_NETBOX_URL=\"\"")
  print("export MANISBLE_NETBOX_TOKEN=\"\"")
  print("export MANISBLE_NETBOX_SSL=\"\"")


def  get_env():
  myenv = {}
  myenv['subproject'] = {}
  try:
    myenv['MANISBLE_NETBOX_URL'] = os.getenv("MANISBLE_NETBOX_URL")
    myenv['MANISBLE_NETBOX_TOKEN'] = os.getenv("MANISBLE_NETBOX_TOKEN")
    myenv['MANISBLE_NETBOX_SSL'] = os.getenv("MANISBLE_NETBOX_SSL", "false")
    myenv['MANISBLE_WORKDIR'] = os.getenv("MANISBLE_WORKDIR", "/tmp/manisble")
  except KeyError as key_error:
    print(key_error)
    usage()
    raise SystemExit("Unable to get environment variables.")
  if myenv['MANISBLE_NETBOX_URL'] == None:
    usage()
    raise SystemExit("Unable to get environment variables.")
  if myenv['MANISBLE_NETBOX_TOKEN'] == None:
    usage()
    raise SystemExit("Unable to get environment variables.")
  
  if myenv['MANISBLE_NETBOX_SSL'] == "false" or myenv['MANISBLE_NETBOX_SSL'] == "False" or myenv['MANISBLE_NETBOX_SSL'] == "FALSE" or myenv['MANISBLE_NETBOX_SSL'] == "no" or myenv['MANISBLE_NETBOX_SSL'] == "NO" or myenv['MANISBLE_NETBOX_SSL'] == "No":
    myenv['MANISBLE_NETBOX_SSL'] = False
  else:
    myenv['MANISBLE_NETBOX_SSL'] = True
  if myenv['MANISBLE_NETBOX_URL'][-1] == "/":
    myenv['MANISBLE_NETBOX_URL'] = myenv['MANISBLE_NETBOX_URL'][:-1]

  

  # list all files in /etc 
  if os.path.exists(myenv['MANISBLE_WORKDIR']) == False:
    os.mkdir(myenv['MANISBLE_WORKDIR'])
  files = os.listdir(myenv['MANISBLE_WORKDIR'] + "/etc/manisble")
  print(files)

  if os.path.exists(myenv['MANISBLE_WORKDIR'] + "/etc/manisble/manisble.json") == False:
    raise SystemExit("Unable to find " + myenv['MANISBLE_WORKDIR'] +"/etc/manisble/manisble.json")
  

  f = open(myenv['MANISBLE_WORKDIR'] + "/etc/manisble/manisble.json", "r")
  manisbleconfig = json.loads(f.read())
  f.close()
  for key in manisbleconfig:
    myenv[key] = manisbleconfig[key]

  mysubprojects = []
  for subproject in myenv['subprojects']:
    filename = myenv['MANISBLE_WORKDIR'] + "/etc/manisble/conf.d/" + subproject['name'] + ".json"
    try: 
      ff =  open(filename, "r")
      ff.close()
    except:
      errorstring = "unable to open " + filename
      print(errorstring)
# create file
#{
#  "subproject": {
#    "description": "The zabbix agent installation and configuration"
#  },
#  "inventory": {
#    "globalvars": {
#      "zabbix_server": "zabbix.it.rm.dk"
#    }
#  },
#  "hosts": [
#    "exrhel001.it.rm.dk",
#    "exrhel002.it.rm.dk"
#  ]
#}
      description = "The %s project" % subproject['name']
      data = {
    "subproject": {
        "description": description
    },
    "inventory": {
        "globalvars": {
        }
    },
    "hosts": [
    ]
}
      with open(filename, "w") as file:
        json.dump(data, file, indent=2)

    ff =  open(filename, "r")
    subprojectconfig = json.loads(ff.read())
    print(subprojectconfig)
    myenv['subproject'][subproject['name']] = subprojectconfig
    ff.close()







  return myenv



import os
import json
import requests
from ..common import prettyllog

from .awx_common import refresh_awx_data
from .awx_common import awx_get_id
from .awx_common import getawxdata

VERIFY_SSL = os.getenv("VERIFY_SSL", "false")
if VERIFY_SSL.lower() == "false" or VERIFY_SSL == "no":
    VERIFY_SSL = False
else:
    VERIFY_SSL = True


######################################
# function: get Project 
######################################
def awx_get_project(projid, organization=None, mytoken=None, r=None):
  headers = {"User-agent": "python-awx-client", "Content-Type": "application/json","Authorization": "Bearer {}".format(mytoken)}
  orgid = (awx_get_id("organizations", organization, r))
  url = os.getenv("TOWER_HOST") + "/api/v2/projects/%s" % projid
  resp = requests.get(url,headers=headers,  verify=VERIFY_SSL)
  return   json.loads(resp.content)



######################################
# function: Create Project 
######################################
def awx_create_project(name, description, scm_type, scm_url, scm_branch, credential, organization, mytoken, r):
  getawxdata("projects", mytoken, r)
  try:  
    projid = (awx_get_id("projects", name, r))
  except:
    print("Unexpected error")
  headers = {"User-agent": "python-awx-client", "Content-Type": "application/json","Authorization": "Bearer {}".format(mytoken)}
  orgid = (awx_get_id("organizations", organization, r))
  credid = (awx_get_id("credentials", credential, r))
  data = {
        "name": name,
        "description": description,
        "scm_type": scm_type,
        "scm_url": scm_url,
        "organization": orgid,
        "scm_branch": scm_branch,
        "credential": credid
       }
  if (projid == ""):
    url = os.getenv("TOWER_HOST") + "/api/v2/projects/"
    resp = requests.post(url,headers=headers, json=data, verify=VERIFY_SSL)
    response = json.loads(resp.content)
    try:
      projid=response['id']
      prettyllog("manage", "project", name, organization, resp.status_code, projid)
    except:
      prettyllog("manage", "project", name, organization, resp.status_code, response)
    #loop until project is synced
    loop = True
    while ( loop ):
        getawxdata("projects", mytoken, r)
        try:  
            projid = (awx_get_id("projects", name, r))
        except:
            print("Unexpected error")
        projectinfo = awx_get_project(projid, organization, mytoken , r)
        try:
          if( projectinfo['status'] == "successful"):
              loop = False
        except:
          print("Project status unknown")

  else:
    url = os.getenv("TOWER_HOST") + "/api/v2/projects/%s/" % projid
    resp = requests.put(url,headers=headers, json=data, verify=VERIFY_SSL)
    response = json.loads(resp.content)
    try:  
      projid = (awx_get_id("projects", name,r ))
      prettyllog("manage", "project", name, organization, resp.status_code, projid)
    except:
      prettyllog("manage", "project", name, organization, resp.status_code, response)
    getawxdata("projects", mytoken, r)
    try:
        projid = (awx_get_id("projects", name, r ))
    except:
        print("Unexpected error")
    projectinfo = awx_get_project(projid, organization, mytoken, r)
    if( projectinfo['status'] == "successful"):
      prettyllog("manage", "project", name, organization, "000", "Project is ready")
    else:    
      prettyllog("manage", "project", name, organization, "666", "Project is not ready")
  refresh_awx_data(mytoken, r)



import requests
import json
import os
import base64
import xml.etree.ElementTree as ET
import platform
import pprint
import yaml
from ..common import prettyllog


def get_orgs(env):
    orgs = {}
    url = env['MANISBLE_NETBOX_URL'] + "/api/tenancy/organizations/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN']}
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        for org in data['results']:
            orgs[org['name']] = org['id']
    else:
        prettyllog("netbox", "get", "orgs", "error", r.status_code , "unable to get orgs", severity="ERROR")
    return orgs



def create_tenant_group(tenant_group_name, env):
    url = env['MANISBLE_NETBOX_URL'] + "/api/tenancy/tenant-groups/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    data = {
        "name": tenant_group_name,
        #strip all wovel characters and replace spaces with -
        "slug": tenant_group_name.lower().replace(" ", "-"),
        "description": "Created by MANISBLE"
    }
    r = requests.post(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 201:
        data = r.json()
        prettyllog("netbox", "create", "tenant_group", tenant_group_name, r.status_code , "tenant_group created", severity="CHANGE")
        return data['id']
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "tenant_group", tenant_group_name, r.status_code , "tenant_group already exists", severity="INFO")
            return True
        prettyllog("netbox", "create", "tenant_group", tenant_group_name, r.status_code , "unable to create tenant_group", severity="ERROR")
        return False
    
def get_tenant_group_id(tenant_group_name, env):
    tenant_groups = get_tenant_groups(env)
    try: 
        return tenant_groups[tenant_group_name]
    except:
        return False
    
def get_tenant_groups(env):
    tenant_groups = {}
    url = env['MANISBLE_NETBOX_URL'] + "/api/tenancy/tenant-groups/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN']}
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        for tenant_group in data['results']:
            tenant_groups[tenant_group['name']] = tenant_group['id']
    else:
        prettyllog("netbox", "get", "tenant_groups", "error", r.status_code , "unable to get tenant_groups", severity="ERROR")
    return tenant_groups

def create_tenant(tenant_name, tenant_group, env):
    tenant_group_id = get_tenant_group_id(tenant_group, env)
    url = env['MANISBLE_NETBOX_URL'] + "/api/tenancy/tenants/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    data = {
        "name": tenant_name,
        "slug": tenant_name.lower(),
        "group": tenant_group_id,
        "description": "Created by MANISBLE"
    }
    r = requests.post(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 201:
        data = r.json()
        return data['id']
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "tenant", tenant_name, r.status_code , "tenant already exists", severity="INFO")
            return True
        prettyllog("netbox", "create", "tenant", tenant_name, r.status_code , "unable to create tenant", severity="ERROR")
        return False
    
def get_tenant_id(tenant_name, env):
    tenants = get_tenants(env)
    try: 
        return tenants[tenant_name]
    except:
        return False
    
def get_tenants(env):
    tenants = {}
    url = env['MANISBLE_NETBOX_URL'] + "/api/tenancy/tenants/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN']}
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        for tenant in data['results']:
            tenants[tenant['name']] = tenant['id']
    else:
        prettyllog("netbox", "get", "tenants", "error", r.status_code , "unable to get tenants", severity="ERROR")
    return tenants

def create_sitegroups(sitegroup_name, env):
    url = env['MANISBLE_NETBOX_URL'] + "/api/dcim/site-groups/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    data = {
        "name": sitegroup_name,
        "slug": sitegroup_name.lower().replace(" ", "-"),
        "description": "Created by MANISBLE"
    }
    r = requests.post(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 201:
        data = r.json()
        return data['id']
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "sitegroup", sitegroup_name, r.status_code , "sitegroup already exists", severity="INFO")
            return True

        prettyllog("netbox", "create", "sitegroup", sitegroup_name, r.status_code , "unable to create sitegroup", severity="ERROR")
        return False
    
def get_sitegroup_id(sitegroup_name, env):
    sitegroups = get_sitegroups(env)
    try: 
        return sitegroups[sitegroup_name]
    except:
        return False
    
def get_sitegroups(env):
    sitegroups = {}
    url = env['MANISBLE_NETBOX_URL'] + "/api/dcim/site-groups/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN']}
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        for sitegroup in data['results']:
            sitegroups[sitegroup['name']] = sitegroup['id']
    else:
        prettyllog("netbox", "get", "sitegroups", "error", r.status_code , "unable to get sitegroups", severity="ERROR")
    return sitegroups

def create_site(site_name, region, sitegroup, tenant, env):
    sitegroup_id = get_sitegroup_id(sitegroup, env)
    region_id = get_region_id(region, env)
    tenant_id = get_tenant_id(tenant, env)

    url = env['MANISBLE_NETBOX_URL'] + "/api/dcim/sites/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    data = {
        "name": site_name,
        "slug": site_name.lower().replace(" ", "-"),
        "region": region_id,
        "group": sitegroup_id,
        "tenant": tenant_id,
        "description": "Created by MANISBLE"
    }
    r = requests.post(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 201:
        data = r.json()
        return data['id']
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "site", site_name, r.status_code , "site already exists", severity="INFO")
            return True

        prettyllog("netbox", "create", "site", site_name, r.status_code , "unable to create site", severity="ERROR")
        return False
    
def get_site_id(site_name, env):
    sites = get_sites(env)
    try: 
        return sites[site_name]
    except:
        return False
    
def get_sites(env):
    sites = {}
    url = env['MANISBLE_NETBOX_URL'] + "/api/dcim/sites/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN']}
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        for site in data['results']:
            sites[site['name']] = site['id']
    else:
        prettyllog("netbox", "get", "sites", "error", r.status_code , "unable to get sites", severity="ERROR")
    return sites


def create_region(region_name, env):
    url = env['MANISBLE_NETBOX_URL'] + "/api/dcim/regions/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    data = {
        "name": region_name,
        "slug": region_name.lower().replace(" ", "-"),
        "description": "Created by MANISBLE"
    }
    r = requests.post(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 201:
        data = r.json()
        return data['id']
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "region", region_name, r.status_code , "region already exists", severity="INFO")
            return True
        prettyllog("netbox", "create", "region", region_name, r.status_code , "unable to create region", severity="ERROR")
        return False
    
def get_region_id(region_name, env):
    regions = get_regions(env)
    try: 
        return regions[region_name]
    except:
        return False
    
def get_regions(env):
    regions = {}
    url = env['MANISBLE_NETBOX_URL'] + "/api/dcim/regions/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN']}
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        for region in data['results']:
            regions[region['name']] = region['id']
    else:
        prettyllog("netbox", "get", "regions", "error", r.status_code , "unable to get regions", severity="ERROR")
    return regions

def read_etc_json():
    try:
        with open('etc/manisble/netbox.json') as f:
            data = json.load(f)
        return data
    except:
        return False
    

def refresh_netbox_orgs(env):
    netboxdata = read_etc_json()
    """
    this is the data structure we are looking for

    {'name': 'Aarhus Universitetshospital',
     'organisation': 'Region Midtjylland',
     'sites': [{'address': 'Palle Juul-Jensens Boulevard 99, 8200 Aarhus N',
            'description': 'Aarhus Universitetshospital',
            'name': 'Aarhus Universitetshospital',
            'region': 'Region Midtjylland',
            'slug': 'auh',
            'stastus': 'Active'}]}

    """
    pprint.pprint(netboxdata)
    
    create_tenant_group(netboxdata['organization'], env)
    create_tenant(netboxdata['name'], netboxdata['organization'], env)
    for site in netboxdata['sites']:
        create_region(site['region'], env)
        create_sitegroups("Kalm", env)
        create_site(site['name'], site['region'], "Kalm", netboxdata['name'], env)
    return netboxdata


    # create tenant groups
    # we need a group for the organisation





from . import bump
import argparse
from ..common import prettyllog




def main():
    parser = argparse.ArgumentParser(description="Keep manisble and bump", usage="manisble_bump <action> \n\n \
               \
               version : 1.0.0 bump  \n                                              \
               actions:\n                                                      \
               major        major bump \n  \
               minor        minor bump \n  \
               patch        patch bump \n  \
               \
               2023 Knowit Miracle\
               ")
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup jenkis')
    args = parser.parse_args()
    ready = False
    print("check if we are ready to go")


    if args.action[0] == "major":
        bump.major()
        ready = True
    if args.action[0] == "minor":
        bump.minor()
        ready = True
    if args.action[0] == "patch":
        bump.patch()
        ready = True

    


"""
Written by Michael Rice <michael@michaelrice.org>
Github: https://github.com/michaelrice
Website: https://michaelrice.github.io/
Blog: http://www.errr-online.com/
This code has been released under the terms of the Apache 2.0 licenses
http://www.apache.org/licenses/LICENSE-2.0.html
"""

import logging
from xml.etree.ElementTree import Element
from xml.etree.ElementTree import SubElement
from xml.etree.ElementTree import tostring

import requests


def reset_alarm(**kwargs):
    """
    Resets an alarm on a given HostSystem in a vCenter to the green state
    without someone having to log in to do it manually.

    This is done by using an unexposed API call. This requires us
    to manually construct the SOAP envelope. We use the session key
    that pyvmomi provides during its connection.

    More information can be found about this process
    in this article written by William Lam:
    http://www.virtuallyghetto.com/2010/10/how-to-ack-reset-vcenter-alarm.html

    I adopted his process from perl to groovy:
    https://gist.github.com/michaelrice/d54a237295e017b032a5
    and from groovy now to python.

    Usage:
    SI = SmartConnect(xxx)
    HOST = SI.content.searchIndex.FindByxxx(xxx)
    alarm.reset_alarm(entity_moref=HOST._moId, entity_type='HostSystem',
                      alarm_moref='alarm-1', service_instance=SI)
    :param service_instance:
    :param entity_moref:
    :param alarm:
    :return boolean:
    """
    service_instance = kwargs.get("service_instance")
    payload = _build_payload(**kwargs)
    logging.debug(payload)
    session = service_instance._stub
    if not _send_request(payload, session):
        return False
    return True


def _build_payload(**kwargs):
    """
    Builds a SOAP envelope to send to the vCenter hidden API

    :param entity_moref:
    :param alarm_moref:
    :param entity_type:
    :return:
    """
    entity_moref = kwargs.get("entity_moref")
    entity_type = kwargs.get("entity_type")
    alarm_moref = kwargs.get("alarm_moref")
    if not entity_moref or not entity_type or not alarm_moref:
        raise ValueError("entity_moref, entity_type, and alarm_moref "
                         "must be set")

    attribs = {
        'xmlns:xsd': 'http://www.w3.org/2001/XMLSchema',
        'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
        'xmlns:soap': 'http://schemas.xmlsoap.org/soap/envelope/'
    }
    root = Element('soap:Envelope', attribs)
    body = SubElement(root, 'soap:Body')
    alarm_status = SubElement(body, 'SetAlarmStatus', {'xmlns': 'urn:vim25'})
    this = SubElement(alarm_status, '_this', {
        'xsi:type': 'ManagedObjectReference',
        'type': 'AlarmManager'
    })
    this.text = 'AlarmManager'
    alarm = SubElement(alarm_status, 'alarm', {'type': 'Alarm'})
    alarm.text = alarm_moref
    entity = SubElement(alarm_status, 'entity', {
        'xsi:type': 'ManagedObjectReference',
        'type': entity_type
    })
    entity.text = entity_moref
    status = SubElement(alarm_status, 'status')
    status.text = 'green'
    # I hate hard coding this but I have no idea how to do it any other way
    # pull requests welcome :)
    return tostring(root, encoding='UTF-8', method='xml').decode()


def _send_request(payload=None, session=None):
    """
    Using requests we send a SOAP envelope directly to the
    vCenter API to reset an alarm to the green state.

    :param payload:
    :param session:
    :return:
    """
    stub = session
    host_port = stub.host
    # Ive seen some code in pyvmomi where it seems like we check for http vs
    # https but since the default is https do people really run it on http?
    url = 'https://{0}/sdk'.format(host_port)
    logging.debug("Sending %s to %s", payload, url)
    # I opted to ignore invalid ssl here because that happens in pyvmomi.
    # Once pyvmomi validates ssl it wont take much to make it happen here.
    res = requests.post(url=url, data=payload, headers={
        'Cookie': stub.cookie,
        'SOAPAction': 'urn:vim25',
        'Content-Type': 'application/xml'
    }, verify=False)
    if res.status_code != 200:
        logging.debug("Failed to reset alarm. HTTP Status: %s", res.status_code)
        return False
    return True


def print_triggered_alarms(entity=None):
    """
    This is a useful method if you need to print out the alarm morefs

    :param entity:
    """
    alarms = entity.triggeredAlarmState
    for alarm in alarms:
        print("#"*40)
        # The alarm key looks like alarm-101.host-95
        print("alarm_moref: {0}".format(alarm.key.split('.')[0]))
        print("alarm status: {0}".format(alarm.overallStatus))


def get_alarm_refs(entity=None):
    """
    Useful method that will return a list of dict with the moref and alarm
    status for all triggered alarms on a given entity.


    :param entity:
    :return list: [{'alarm':'alarm-101', 'status':'red'}]
    """
    alarm_states = entity.triggeredAlarmState
    ret = []
    for alarm_state in alarm_states:
        tdict = {
            "alarm": alarm_state.key.split('.')[0],
            "status": alarm_state.overallStatus
        }
        ret.append(tdict)
    return ret



import requests
import json
import os
import base64
import xml.etree.ElementTree as ET
import platform
import pprint
import yaml
from ..common import prettyllog
from .organization import get_site_id



def get_cluster_group_id(name, env):
    prettyllog("netbox", "get", "cluster group", name, "000" , "getting cluster group", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/cluster-groups/?name=" + name
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        if len(data['results']) == 1:
            prettyllog("netbox", "get", "cluster group", name, r.status_code , "cluster group found", severity="INFO")
            return data['results'][0]['id']
        else:
            prettyllog("netbox", "get", "cluster group", name, r.status_code , "cluster group not found", severity="INFO")
            return False
    else:
        prettyllog("netbox", "get", "cluster group", name, r.status_code , "unable to get cluster group", severity="ERROR")
        return False

def create_cluster_group(name, env):
    prettyllog("netbox", "create", "cluster group", name, "000" , "creating cluster group", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/cluster-groups/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    payload = {
        "name": name,
        "slug": name,
        "comments": "Created by MANISBLE"
    }
    r = requests.post(url, headers=headers, json=payload, verify=env['MANISBLE_NETBOX_SSL'])
    pprint.pprint(r.content)
    if r.status_code == 201:
        data = r.json()
        prettyllog("netbox", "create", "cluster group", name, r.status_code , "cluster group created", severity="INFO")
        return data['id']
    else:
        prettyllog("netbox", "create", "cluster group", name, r.status_code , "u2nable to create cluster group", severity="ERROR")
        return False
    
def get_clusters(env):
    clusters = {}
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/clusters/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        for cluster in data['results']:
            clusters[cluster['name']] = cluster['id']
    else:
        prettyllog("netbox", "get", "clusters", "error", r.status_code , "unable to get clusters", severity="ERROR")
    return clusters

def get_cluster_id(cluster_name, env):
    clusters = get_clusters(env)
    try: 
        return clusters[cluster_name]
    except:
        return False

def get_cluster_type_id(cluster_type, env, netboxdata):
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/cluster-types/?name=" + cluster_type
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        if len(data['results']) == 1:
            return data['results'][0]['id']
        else:
            return False
    else:
        prettyllog("netbox", "get", "cluster type", cluster_type, r.status_code , "unable to get cluster type", severity="ERROR")
        return False
    

def create_cluster_type(cluster_type, env, netboxdata):
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/cluster-types/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    payload = {
        "name": cluster_type,
        "slug": cluster_type.lower().replace(" ", "-"),
        "comments": "Created by MANISBLE"
    }
    r = requests.post(url, headers=headers, json=payload, verify=env['MANISBLE_NETBOX_SSL'])
    pprint.pprint(r.content)
    if r.status_code == 201:
        data = r.json()
        return data['id']
    else:
        if r.status_code == 400:
            return True
        prettyllog("netbox", "create", "cluster type", cluster_type, r.status_code , "unable to create cluster type", severity="ERROR")
        return False
    
    


    
def create_cluster(cluster_name, clustergroup, env, netboxdata):
    prettyllog("netbox", "create", "cluster", cluster_name, "000" , "creating cluster step 1", severity="INFO")
    if not get_cluster_group_id(clustergroup, env):
        create_cluster_group(clustergroup, env)
    clustergroup_id = get_cluster_group_id(clustergroup, env)
    clustertype = "VMware vCenter"
    if not get_cluster_type_id(clustertype, env, netboxdata):
        create_cluster_type(clustertype, env, netboxdata)
    clustertype_id = get_cluster_type_id(clustertype, env, netboxdata)

    if not clustergroup_id:
        prettyllog("netbox", "create", "cluster", cluster_name, "000" , "unable to create cluster group", severity="ERROR")
        return False

    cluster_id = get_cluster_id(cluster_name, env)  
    if cluster_id:
        prettyllog("netbox", "create", "cluster", cluster_name, "000" , "cluster already exists", severity="INFO")
        return True
    prettyllog("netbox", "create", "cluster", cluster_name, "000" , "creating cluster step 2", severity="INFO")
    siteid = get_site_id(netboxdata['sites'][0]['name'], env)
    prettyllog("netbox", "create", "cluster", siteid, "000" , "creating cluster must relate to this site id", severity="INFO")


    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/clusters/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    data = {
        "name": cluster_name,
        "type": clustertype_id,
        "group": clustergroup_id,
        "site": siteid,
        "comments": "Created by MANISBLE"
    }
    r = requests.post(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    pprint.pprint(r.content)
    if r.status_code == 201:
        data = r.json()
        return data['id']
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "cluster", cluster_name, r.status_code , "cluster already exists", severity="INFO")
            return True
        prettyllog("netbox", "create", "cluster", cluster_name, r.status_code , "u1nable to create cluster 1", severity="ERROR")
        return False

def update_cluster(cluster_name, cluster_type, group, site, env, netboxdata):
    cluster_id = get_cluster_id(cluster_name, env)
    siteid = get_site_id(netboxdata['name'], env)
    clustergroup_id = get_cluster_group_id(clustergroup, env)
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/clusters/" + str(cluster_id) + "/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }    
    data = {
        "name": cluster_name,
        "type": 1,
        "group": clustergroup_id,
        "site": siteid,
        "comments": "Updated by MANISBLE"
    }
    r = requests.put(url, headers=headers, data=data, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        return data['id']
    else:
        prettyllog("netbox", "update", "cluster", cluster_name, r.status_code , "unable to update cluster", severity="ERROR")
        return False


import atexit
from getpass import getpass

from pyVim import connect

"""
This module overlays the pyVmomi library to make its use in a
python shell or short program more enjoyable.
Starting point is instantiating a vCenter Host (VVC) in order
to get all VMs.
"""


class VVC(object):
    """
    A vCenter host.
    """

    def __init__(self, hostname):
        """
        Creates a VVC instance.

        - `hostname` (str) is the name of the vCenter host.
        """
        self.hostname = hostname

    def connect(self, username, password=None):
        """
        Connects to the vCenter host encapsulated by this VVC instance.

        - `username` (str) is the username to use for authentication.
        - `password` (str) is the password to use for authentication.
          If the password is not specified, a getpass prompt will be used.
        """
        if not password:
            password = getpass("Password for {0}: ".format(self.hostname))
        self.service_instance = connect.SmartConnect(host=self.hostname,
                                                     user=username,
                                                     pwd=password,
                                                     port=443)
        atexit.register(connect.Disconnect, self.service_instance)

    def get_first_level_of_vm_folders(self):
        content = self.service_instance.RetrieveContent()
        children = content.rootFolder.childEntity
        for child in children:
            if hasattr(child, "vmFolder"):
                yield child.vmFolder

    def get_all_vms(self):
        """
        Returns a generator over all VMs known to this vCenter host.
        """
        for folder in self.get_first_level_of_vm_folders():
            for vm in get_all_vms_in_folder(folder):
                yield vm


class ESX(object):
    """
    An ESX instance.
    """

    def __init__(self, raw_esx):
        self.raw_esx = raw_esx
        self.name = raw_esx.name

    def __eq__(self, other):
        return self.name == other.name

    def __hash__(self):
        return int("".join((str(ord(c)) for c in self.name)))

    def __getattr__(self, attribute):
        return getattr(self.raw_esx, attribute)

    def get_number_of_cores(self):
        """
        Returns the number of CPU cores (type long) on this ESX.
        """
        resources_on_esx = self.raw_esx.licensableResource.resource
        for resource in resources_on_esx:
            if resource.key == "numCpuCores":
                return resource.value
        message = "{0} has no resource numCpuCores.\n Available resources: {1}"
        raise RuntimeError(message.format(self.name, resources_on_esx))


class VM(object):
    """
    A virtual machine.
    """

    def __init__(self, raw_vm):
        self.raw_vm = raw_vm
        self.name = raw_vm.name

    def __getattr__(self, attribute):
        return getattr(self.raw_vm, attribute)

    def get_first_network_interface_matching(self, predicate):
        """
        Returns the first network interface of this VM that matches the given
        predicate.

        - `predicate` (callable) is a function that takes a network and returns
          True (return this network) or False (skip this network).
        """
        for network in self.raw_vm.network:
            if predicate(network):
                return network
        return None

    def get_esx_host(self):
        return ESX(self.raw_vm.runtime.host)


def get_all_vms_in_folder(folder):
    vm_or_folders = folder.childEntity
    for vm_or_folder in vm_or_folders:
        if hasattr(vm_or_folder, "childEntity"):
            # it's still a folder, look deeper
            for vm in get_all_vms_in_folder(vm_or_folder):
                yield vm  # it's now a VM
        else:
            yield VM(vm_or_folder)  # it's a VM



# VMware vSphere Python SDK Community Samples Addons
# Copyright (c) 2014-2021 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module implements simple helper functions for python samples working with
virtual machine objects
"""
__author__ = "VMware, Inc."


def print_vm_info(vm, depth=1, max_depth=10):
    """
    Print information for a particular virtual machine or recurse into a
    folder with depth protection
    """

    # if this is a group it will have children. if it does, recurse into them
    # and then return
    if hasattr(vm, 'childEntity'):
        if depth > max_depth:
            return
        vm_list = vm.childEntity
        for child_vm in vm_list:
            print_vm_info(child_vm, depth + 1)
        return

    summary = vm.summary
    print("Name       :", summary.config.name)
    print("Path       :", summary.config.vmPathName)
    print("Guest      :", summary.config.guestFullName)
    annotation = summary.config.annotation
    if annotation:
        print("Annotation :", annotation)
    print("State      :", summary.runtime.powerState)
    if summary.guest is not None:
        ip = summary.guest.ipAddress
        if ip:
            print("IP         :", ip)
    if summary.runtime.question is not None:
        print("Question  :", summary.runtime.question.text)
    print("")



# Written by Chris Arceneaux
# GitHub: https://github.com/carceneaux
# Email: carceneaux@thinksis.com
# Website: http://arsano.ninja
#
# This code has been released under the terms of the Apache-2.0 license
# http://opensource.org/licenses/Apache-2.0

"""
This module implements simple helper functions for working with the
VMware Storage Policy (pbm) API
"""

from pyVmomi import pbm, VmomiSupport


def create_pbm_session(stub):
    """
    Creates a session with the VMware Storage Policy API

    Sample Usage:

    create_pbm_session(service_instance._stub)
    """
    import pyVmomi
    import ssl
    # Make compatible with both Python2/3
    try:
        from http import cookies
    except ImportError:
        import Cookie as cookies

    session_cookie = stub.cookie.split('"')[1]
    http_context = VmomiSupport.GetHttpContext()
    cookie = cookies.SimpleCookie()
    cookie["vmware_soap_session"] = session_cookie
    http_context["cookies"] = cookie
    VmomiSupport.GetRequestContext()["vcSessionCookie"] = session_cookie
    hostname = stub.host.split(":")[0]

    context = None
    if hasattr(ssl, "_create_unverified_context"):
        context = ssl._create_unverified_context()
    pbm_stub = pyVmomi.SoapStubAdapter(
        host=hostname,
        version="pbm.version.version1",
        path="/pbm/sdk",
        poolSize=0,
        sslContext=context)
    pbm_si = pbm.ServiceInstance("ServiceInstance", pbm_stub)

    return pbm_si


def retrieve_storage_policy(pbm_content, policy):
    """
    Retrieves the managed object for the storage policy specified

    Sample Usage:

    pbm_content = pbm_si.RetrieveContent()
    retrieve_storage_policy(pbm_content, "Policy Name")
    """
    # Set PbmQueryProfile
    profile_manager = pbm_content.profileManager

    # Retrieving Storage Policies
    profile_ids = profile_manager.PbmQueryProfile(resourceType=pbm.profile.ResourceType(
        resourceType="STORAGE"), profileCategory="REQUIREMENT"
    )
    if len(profile_ids) > 0:
        profiles = profile_manager.PbmRetrieveContent(profileIds=profile_ids)
    else:
        raise RuntimeError("No Storage Policies found.")

    # Searching for Storage Policy
    storage_polity_profile = None
    for profile in profiles:
        if profile.name == policy:
            storage_polity_profile = profile
            break
    if not storage_polity_profile:
        raise RuntimeError("Storage Policy specified not found.")

    return storage_polity_profile



# VMware vSphere Python SDK Community Samples Addons
# Copyright (c) 2014-2021 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module implements simple helper functions for python samples
"""
import argparse
import getpass

__author__ = "VMware, Inc."


class Parser:
    """
    Samples specific argument parser.
    Wraps argparse to ease the setup of argument requirements for the samples.

    Example:
        parser = cli.Parser()
        parser.add_required_arguments(cli.Argument.VM_NAME)
        parser.add_optional_arguments(cli.Argument.DATACENTER_NAME, cli.Argument.NIC_NAME)
        parser.add_custom_argument(
            '--disk-number', required=True, help='Disk number to change mode.')
        args = parser.get_args()
    """

    def __init__(self):
        """
        Defines two arguments groups.
        One for the standard arguments and one for sample specific arguments.
        The standard group cannot be extended.
        """
        self._parser = argparse.ArgumentParser(description='Arguments for talking to vCenter')
        self._standard_args_group = self._parser.add_argument_group('standard arguments')
        self._specific_args_group = self._parser.add_argument_group('sample-specific arguments')

        # because -h is reserved for 'help' we use -s for service
        self._standard_args_group.add_argument('-s', '--host',
                                               required=True,
                                               action='store',
                                               help='vSphere service address to connect to')

        # because we want -p for password, we use -o for port
        self._standard_args_group.add_argument('-o', '--port',
                                               type=int,
                                               default=443,
                                               action='store',
                                               help='Port to connect on')

        self._standard_args_group.add_argument('-u', '--user',
                                               required=True,
                                               action='store',
                                               help='User name to use when connecting to host')

        self._standard_args_group.add_argument('-p', '--password',
                                               required=False,
                                               action='store',
                                               help='Password to use when connecting to host')

        self._standard_args_group.add_argument('-nossl', '--disable-ssl-verification',
                                               required=False,
                                               action='store_true',
                                               help='Disable ssl host certificate verification')

    def get_args(self):
        """
        Supports the command-line arguments needed to form a connection to vSphere.
        """
        args = self._parser.parse_args()
        return self._prompt_for_password(args)

    def _add_sample_specific_arguments(self, is_required: bool, *args):
        """
        Add an argument to the "sample specific arguments" group
        Requires a predefined argument from the Argument class.
        """
        for arg in args:
            name_or_flags = arg["name_or_flags"]
            options = arg["options"]
            options["required"] = is_required
            self._specific_args_group.add_argument(*name_or_flags, **options)

    def add_required_arguments(self, *args):
        """
        Add a required argument to the "sample specific arguments" group
        Requires a predefined argument from the Argument class.
        """
        self._add_sample_specific_arguments(True, *args)

    def add_optional_arguments(self, *args):
        """
        Add an optional argument to the "sample specific arguments" group.
        Requires a predefined argument from the Argument class.
        """
        self._add_sample_specific_arguments(False, *args)

    def add_custom_argument(self, *name_or_flags, **options):
        """
        Uses ArgumentParser.add_argument() to add a full definition of a command line argument
        to the "sample specific arguments" group.
        https://docs.python.org/3/library/argparse.html#the-add-argument-method
        """
        self._specific_args_group.add_argument(*name_or_flags, **options)

    def set_epilog(self, epilog):
        """
        Text to display after the argument help
        """
        self._parser.epilog = epilog

    def _prompt_for_password(self, args):
        """
        if no password is specified on the command line, prompt for it
        """
        if not args.password:
            args.password = getpass.getpass(
                prompt='"--password" not provided! Please enter password for host %s and user %s: '
                       % (args.host, args.user))
        return args


class Argument:
    """
    Predefined arguments to use in the Parser

    Example:
        parser = cli.Parser()
        parser.add_optional_arguments(cli.Argument.VM_NAME)
        parser.add_optional_arguments(cli.Argument.DATACENTER_NAME, cli.Argument.NIC_NAME)
    """
    def __init__(self):
        pass

    UUID = {
        'name_or_flags': ['--uuid'],
        'options': {'action': 'store', 'help': 'UUID of an entity (VirtualMachine or HostSystem)'}
    }
    VM_NAME = {
        'name_or_flags': ['-v', '--vm-name'],
        'options': {'action': 'store', 'help': 'Name of the vm'}
    }
    VM_IP = {
        'name_or_flags': ['--vm-ip'],
        'options': {'action': 'store', 'help': 'IP of the vm'}
    }
    VM_MAC = {
        'name_or_flags': ['-mac', '--vm-mac'],
        'options': {'action': 'store', 'help': 'Mac address of the VM'}
    }
    VM_USER = {
        'name_or_flags': ['--vm-user'],
        'options': {'action': 'store', 'help': 'virtual machine user name'}
    }
    VM_PASS = {
        'name_or_flags': ['--vm-password'],
        'options': {'action': 'store', 'help': 'virtual machine password'}
    }
    ESX_NAME = {
        'name_or_flags': ['-e', '--esx-name'],
        'options': {'action': 'store', 'help': 'Esx name'}
    }
    ESX_IP = {
        'name_or_flags': ['--esx-ip'],
        'options': {'action': 'store', 'help': 'Esx ip'}
    }
    ESX_NAME_REGEX = {
        'name_or_flags': ['--esx-name-regex'],
        'options': {'action': 'store', 'help': 'Esx name regex'}
    }
    DNS_NAME = {
        'name_or_flags': ['--dns-name'],
        'options': {'action': 'store', 'help': 'DNS name'}
    }
    NAME = {
        'name_or_flags': ['-n', '--name'],
        'options': {'action': 'store', 'help': 'Name of the entity'}
    }
    NEW_NAME = {
        'name_or_flags': ['-r', '--new-name'],
        'options': {'action': 'store', 'help': 'New name of the entity.'}
    }
    DATACENTER_NAME = {
        'name_or_flags': ['--datacenter-name'],
        'options': {'action': 'store', 'help': 'Datacenter name'}
    }
    DATASTORE_NAME = {
        'name_or_flags': ['--datastore-name'],
        'options': {'action': 'store', 'help': 'Datastore name'}
    }
    CLUSTER_NAME = {
        'name_or_flags': ['--cluster-name'],
        'options': {'action': 'store', 'help': 'Cluster name'}
    }
    FOLDER_NAME = {
        'name_or_flags': ['--folder-name'],
        'options': {'action': 'store', 'help': 'Folder name'}
    }
    TEMPLATE = {
        'name_or_flags': ['--template'],
        'options': {'action': 'store', 'help': 'Name of the template/VM'}
    }
    VMFOLDER = {
        'name_or_flags': ['--vm-folder'],
        'options': {'action': 'store', 'help': 'Name of the VMFolder'}
    }
    DATASTORECLUSTER_NAME = {
        'name_or_flags': ['--datastorecluster-name'],
        'options': {'action': 'store', 'help': 'Datastorecluster (DRS Storagepod)'}
    }
    RESOURCE_POOL = {
        'name_or_flags': ['--resource-pool'],
        'options': {'action': 'store', 'help': 'Resource pool name'}
    }
    POWER_ON = {
        'name_or_flags': ['--power-on'],
        'options': {'action': 'store_true', 'help': 'power on the VM'}
    }
    LANGUAGE = {
        'name_or_flags': ['--language'],
        'options': {'action': 'store', 'default': 'English', 'help': 'Language your vcenter used.'}
    }
    VIHOST = {
        'name_or_flags': ['--vihost'],
        'options': {'action': 'store',
                    'help': 'Name/ip address of ESXi host as seen in vCenter Server'}
    }
    DVS_PORT_GROUP_NAME = {
        'name_or_flags': ['--dvs-pg-name'],
        'options': {'action': 'store', 'help': '"Name of the distributed port group'}
    }
    DVS_NAME = {
        'name_or_flags': ['--dvs-name'],
        'options': {'action': 'store', 'help': 'Name of the distributed virtual switch'}
    }
    OPAQUE_NETWORK_NAME = {
        'name_or_flags': ['--opaque-network-name'],
        'options': {'action': 'store', 'help': 'Name of an opaque network'}
    }
    FIRST_CLASS_DISK_NAME = {
        'name_or_flags': ['--fcd-name'],
        'options': {'action': 'store', 'help': 'First Class Disk name'}
    }
    DISK_TYPE = {
        'name_or_flags': ['--disk-type'],
        'options': {'action': 'store',
                    'default': 'thin', 'choices': ['thick', 'thin'], 'help': 'thick or thin'}
    }
    DISK_SIZE = {
        'name_or_flags': ['--disk-size'],
        'options': {'action': 'store', 'help': 'disk size, in GB, to add to the VM'}
    }
    PORT_GROUP = {
        'name_or_flags': ['-g', '--port-group'],
        'options': {'action': 'store', 'help': 'Name of port group'}
    }
    NETWORK_NAME = {
        'name_or_flags': ['--network-name'],
        'options': {'action': 'store', 'help': 'Name of network'}
    }
    VSWITCH_NAME = {
        'name_or_flags': ['-w', '--vswitch-name'],
        'options': {'action': 'store', 'help': 'vSwitch name'}
    }
    LOCAL_FILE_PATH = {
        'name_or_flags': ['--local-file-path'],
        'options': {'action': 'store', 'help': 'Local disk path to file'}
    }
    REMOTE_FILE_PATH = {
        'name_or_flags': ['--remote-file-path'],
        'options': {'action': 'store', 'help': 'Path on datastore or vm or other entity to file'}
    }
    VLAN_ID = {
        'name_or_flags': ['--vlan-id'],
        'options': {'action': 'store', 'help': 'Vlan ID'}
    }
    DEVICE_NAME = {
        'name_or_flags': ['--device-name'],
        'options': {'action': 'store', 'help': 'The device name. Might look like '
                                               '"/vmfs/devices/disks/naa.*". '
                                               'See vim.vm.device.VirtualDisk.'
                                               'RawDiskMappingVer1BackingInfo documentation.'}}
    DISK_MODE = {
        'name_or_flags': ['--disk-mode'],
        'options': {'action': 'store',
                    'default': 'independent_persistent',
                    'choices': [
                        'append',
                        'independent_nonpersistent',
                        'independent_persistent',
                        'nonpersistent',
                        'persistent',
                        'undoable'],
                    'help': 'See vim.vm.device.VirtualDiskOption.DiskMode documentation.'}}

    COMPATIBILITY_MODE = {
        'name_or_flags': ['--disk-compatibility-mode'],
        'options': {'action': 'store',
                    'default': 'virtualMode',
                    'choices': ['physicalMode', 'virtualMode'],
                    'help': 'See vim.vm.device.VirtualDiskOption.CompatibilityMode documentation.'}}

    ISO = {
        'name_or_flags': ['--iso'],
        'options': {'action': 'store',
                    'help': 'ISO to use in test. Use datastore path format. '
                            'E.g. [datastore1] path/to/file.iso'}
    }
    NIC_NAME = {
        'name_or_flags': ['--nic-name'],
        'options': {'action': 'store', 'help': 'NIC number.'}
    }
    NIC_UNIT_NUMBER = {
        'name_or_flags': ['--nic-unitnumber'],
        'options': {'action': 'store', 'type': int, 'help': 'NIC number.'}
    }
    NIC_STATE = {
        'name_or_flags': ['--nic-state'],
        'options': {'action': 'store', 'choices': ['delete', 'disconnect', 'connect'],
                    'help': 'NIC number.'}
    }
    VMDK_PATH = {
        'name_or_flags': ['--vmdk-path'],
        'options': {'action': 'store', 'help': 'Path of the VMDK file.'}
    }
    OVA_PATH = {
        'name_or_flags': ['--ova-path'],
        'options': {'action': 'store', 'help': 'Path to the OVA file.'}
    }
    OVF_PATH = {
        'name_or_flags': ['--ovf-path'],
        'options': {'action': 'store', 'help': 'Path to the OVF file.'}
    }
    DATE = {
        'name_or_flags': ['--date'],
        'options': {'action': 'store', 'help': 'Date and time with the format d/m/Y H:M'}
    }
    MINUTES = {
        'name_or_flags': ['--minutes'],
        'options': {'action': 'store', 'help': 'time in minutes'}
    }
    MESSAGE = {
        'name_or_flags': ['-m', '--message'],
        'options': {'action': 'store', 'help': 'Message'}
    }

    SNAPSHOT_OPERATION = {
        'name_or_flags': ['-op', '--snapshot-operation'],
        'options': {'action': 'store',
                    'choices':
                        ['create', 'remove', 'revert', 'list_all', 'list_current', 'remove_all'],
                    'help': 'Snapshot operation'}
    }
    SNAPSHOT_NAME = {
        'name_or_flags': ['--snapshot-name'],
        'options': {'action': 'store', 'help': 'Snapshot name'}
    }
    STORAGE_POLICY_NAME = {
        'name_or_flags': ['--storage-policy-name'],
        'options': {'action': 'store', 'metavar': 'string', 'help': 'Storage policy name'}
    }
    ASSUME_INPUT = {
        'name_or_flags': ['--assume-input'],
        'options': {'action': 'store', 'help': 'Assume user input'}
    }
    SSL_KEY = {
        'name_or_flags': ['--ssl-key'],
        'options': {'action': 'store', 'help': 'absolute location of the private key file'}
    }
    SSL_CERT = {
        'name_or_flags': ['--ssl-cert'],
        'options': {'action': 'store', 'help': 'absolute location of the certificate file'}
    }


def prompt_y_n_question(question, default="no"):
    """ based on:
        http://code.activestate.com/recipes/577058/
    :param question: Question to ask
    :param default: No
    :return: True/False
    """
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("Invalid default answer: '{}'".format(default))

    while True:
        print(question + prompt)
        choice = input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            print("Please, respond with 'yes' or 'no' or 'y' or 'n'.")



# keep manisble and con5entrate on your semaphores

from . import serve
import argparse

def main():
    parser = argparse.ArgumentParser(description="Keep manisble and con5entrate on your semaphores", usage="manisble_semaphore <action> \n\n \
\
version : 0.0.2 (semaphore)\n\
actions:\n\
serve      keep manisble and serve semaphore\n\
init       keep manisble and init semaphore systemd service\n\
start      keep manisble and start semaphore systemd service\n\
stop       keep manisble and stop semaphore systemd service\n\
restart    keep manisble and restart semaphore systemd service\n\
setup      keep manisble and setup semaphore\n\
test       keep manisble and test semaphore\n\
audit      keep manisble and audit semaphore\n\
\n\
\
2023 Knowit Miracle\
")
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup jenkis')
    args = parser.parse_args()
    ready = False

    if args.action[0] == "serve":
        serve.main()
        return 0
    




import pprint
from ..common import prettyllog
import requests
from .organization import get_site_id
from .organization import get_tenant_id
import json
import pprint
import time


def create_ip_address_role(name, env):
    prettyllog("netbox", "create", "ip address role", name, "000" , "creating ip address role", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/ipam/roles/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    payload = {
        "name": name,
        "slug": name,
        "weight": 100
    }
    r = requests.post(url, headers=headers, data=json.dumps(payload), verify=env['MANISBLE_NETBOX_SSL'])
    pprint.pprint(r.content)
    if r.status_code == 201:
        prettyllog("netbox", "create", "ip address role", name, r.status_code , "ip address role created", severity="INFO")
        return True
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "ip address role", name, r.status_code , "ip address role already exists", severity="INFO")
            return True
        prettyllog("netbox", "create", "ip address role", name, r.status_code , "unable to create ip address role", severity="ERROR")
        return False
    
def  get_ip_address_role_id(name, env):
    prettyllog("netbox", "get", "ip address role", name, "000" , "getting ip address role", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/ipam/roles/?name=" + name
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        if len(data['results']) == 1:
            prettyllog("netbox", "get", "ip address role", name, r.status_code , "ip address role found", severity="INFO")
            return data['results'][0]['id']
        else:
            prettyllog("netbox", "get", "ip address role", name, r.status_code , "ip address role not found", severity="INFO")
            return False
    else:
        prettyllog("netbox", "get", "ip address role", name, r.status_code , "unable to get ip address role", severity="ERROR")
        return False

def create_ip_address_status(name, env):
    prettyllog("netbox", "create", "ip address status", name, "000" , "creating ip address status", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/ipam/statuses/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    payload = {
        "name": name,
        "slug": name,
        "color": "ff0000"
    }
    r = requests.post(url, headers=headers, data=json.dumps(payload), verify=env['MANISBLE_NETBOX_SSL'])
    pprint.pprint(r.content)
    if r.status_code == 201:
        prettyllog("netbox", "create", "ip address status", name, r.status_code , "ip address status created", severity="INFO")
        return True
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "ip address status", name, r.status_code , "ip address status already exists", severity="INFO")
            return True
        prettyllog("netbox", "create", "ip address status", name, r.status_code , "unable to create ip address status", severity="ERROR")
        return False
    
def get_ip_address_status_id(name, env):
    prettyllog("netbox", "get", "ip address status", name, "000" , "getting ip address status", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/ipam/statuses/?name=" + name
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        if len(data['results']) == 1:
            prettyllog("netbox", "get", "ip address status", name, r.status_code , "ip address status found", severity="INFO")
            return data['results'][0]['id']
        else:
            prettyllog("netbox", "get", "ip address status", name, r.status_code , "ip address status not found", severity="INFO")
            return False
    else:
        prettyllog("netbox", "get", "ip address status", name, r.status_code , "unable to get ip address status", severity="ERROR")
        return False
    

def get_ip_address_id(ip, env):
    if ip == "" or ip == None:
        return False
    prettyllog("netbox", "get", "ip address", ip, "000" , "getting ip address", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/ipam/ip-addresses/?address=" + ip
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        if len(data['results']) == 1:
            prettyllog("netbox", "get", "ip address", ip, r.status_code , "ip address found", severity="INFO")
            return data['results'][0]['id']
        else:
            prettyllog("netbox", "get", "ip address", ip, r.status_code , "ip address not found", severity="INFO")
            return False
    else:
        prettyllog("netbox", "get", "ip address", ip, r.status_code , "unable to get ip address", severity="ERROR")
        return False

def create_ip_address(ip, env, netboxdata):
    if ip == "" or ip == None:
        return False
    prettyllog("netbox", "create", "ip address", ip, "000" , "creating ip address", severity="INFO")
    tenant = netboxdata['name']
    tenantid = get_tenant_id(tenant,env)
    url = env['MANISBLE_NETBOX_URL'] + "/api/ipam/ip-addresses/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    payload = {
        "address": ip,
        "tenant": tenantid,
        "dns_name": ip,
        "description": ip,
        "nat_inside": None,
        "nat_outside": None,
        "interface": None,
        "vrf": None,
        "tags": []
    }
    r = requests.post(url, headers=headers, data=json.dumps(payload), verify=env['MANISBLE_NETBOX_SSL'])
    pprint.pprint(r.content.decode("utf-8"))
    if r.status_code == 201:
        prettyllog("netbox", "create", "ip address", ip, r.status_code , "ip address created", severity="INFO")
        return True
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "ip address", ip, r.status_code , "ip address already exists", severity="INFO")
            return True
        prettyllog("netbox", "create", "ip address", ip, r.status_code , "unable to create ip address", severity="ERROR")
        return False
    
def remove_ip_address(ip, env):
    prettyllog("netbox", "delete", "ip address", ip, "000" , "deleting ip address", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/ipam/ip-addresses/" + str(ip) + "/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    r = requests.delete(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 204:
        prettyllog("netbox", "delete", "ip address", ip, r.status_code , "ip address deleted", severity="INFO")
        return True
    else:
        prettyllog("netbox", "delete", "ip address", ip, r.status_code , "unable to delete ip address", severity="ERROR")
        return False
    

    


def create_ip_network(cidr, env):
    prettyllog("netbox", "create", "ip network", cidr, "000" , "creating ip network", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/ipam/prefixes/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    payload = {
        "prefix": cidr,
        "status": 1,
        "role": 1,
        "site": get_site_id(env),
        "tenant": 1
    }
    r = requests.post(url, headers=headers, data=json.dumps(payload), verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 201:
        prettyllog("netbox", "create", "ip network", cidr, r.status_code , "ip network created", severity="INFO")
        return True
    else:
        prettyllog("netbox", "create", "ip network", cidr, r.status_code , "unable to create ip network", severity="ERROR")
        return False

def get_ip_network_id(cidr, env):
    prettyllog("netbox", "get", "ip network id", cidr, "000" , "getting ip network id", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/ipam/prefixes/?prefix=" + cidr
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        if len(data['results']) == 1:
            prettyllog("netbox", "get", "ip network id", cidr, r.status_code , "ip network id found", severity="INFO")
            return data['results'][0]['id']
        else:
            prettyllog("netbox", "get", "ip network id", cidr, r.status_code , "ip network id not found", severity="INFO")
            return False
    else:
        prettyllog("netbox", "get", "ip network id", cidr, r.status_code , "unable to get ip network id", severity="ERROR")
        return False
    


import requests
import json
import os
import base64
import xml.etree.ElementTree as ET
import platform
import pprint
import yaml
from ..common import prettyllog
from .organization import get_site_id
from .clusters import create_cluster,  get_cluster_id
from .netbox_ipaddress import create_ip_address 
from .netbox_ipaddress import get_ip_address_id



def get_virtual_server(servername, env):
    prettyllog("netbox", "get", "virtual server", servername, "000" , "getting virtual server", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/virtual-machines/?name=" + servername
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        if len(data['results']) == 1:
            prettyllog("netbox", "get", "virtual server", servername, r.status_code , "virtual server found", severity="INFO")
            return data['results'][0]['id']
        else:
            prettyllog("netbox", "get", "virtual server", servername, r.status_code , "virtual server not found", severity="INFO")
            return False
    else:
        prettyllog("netbox", "get", "virtual server", servername, r.status_code , "unable to get virtual server", severity="ERROR")
        return False
    
def get_virtual_server_by_id(serverid, env):
    prettyllog("netbox", "get", "virtual server", serverid, "000" , "getting virtual server", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/virtual-machines/" + str(serverid) + "/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        prettyllog("netbox", "get", "virtual server", serverid, r.status_code , "virtual server found", severity="INFO")
        return data
    else:
        prettyllog("netbox", "get", "virtual server", serverid, r.status_code , "unable to get virtual server", severity="ERROR")
        return False

def get_virtual_server_id(servername, env):
    prettyllog("netbox", "get", "virtual server id", servername, "000" , "getting virtual server", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/virtual-machines/?name=" + servername
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        if len(data['results']) == 1:
            prettyllog("netbox", "get", "virtual server", servername, r.status_code , "virtual server found", severity="INFO")
            return data['results'][0]['id']
        else:
            prettyllog("netbox", "get", "virtual server", servername, r.status_code , "virtual server not found", severity="INFO")
            return False
    else:
        prettyllog("netbox", "get", "virtual server", servername, r.status_code , "unable to get virtual server", severity="ERROR")
        return False
    

def create_tag(tagname, env):
    prettyllog("netbox", "create", "tag", tagname, "000" , "creating tag", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/extras/tags/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    data = {
        "name": tagname,
        "slug": tagname.lower().replace(" ", "-"),
        "color": "ff0000",
        "comments": "Created by MANISBLE"
        }
    r = requests.post(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 201:
        data = r.json()
        prettyllog("netbox", "create", "tag", tagname, r.status_code , "tag created", severity="CHANGE")
        return data['id']
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "tag", tagname, r.status_code , "tag exists", severity="INFO")
            return True
        prettyllog("netbox", "create", "tag", tagname, r.status_code , "unable to create tag", severity="ERROR")
        return False
    
def get_virtual_server_tags(serverid, env):
    prettyllog("netbox", "get", "virtual server tags", serverid, "000" , "getting virtual server tags", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/virtual-machines/" + str(serverid)
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    mytags = []
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        pprint.pprint(data)

        for tag in data['tags']:
            mytags.append(tag['name'])
        prettyllog("netbox", "get", "virtual server tags", serverid, r.status_code , "virtual server tags found", severity="INFO")
        return mytags
    else:
        prettyllog("netbox", "get", "virtual server tags", serverid, r.status_code , "unable to get virtual server tags", severity="ERROR")
        pprint.pprint(r.content)
        return mytags
    
def get_all_tags(env):
    prettyllog("netbox", "get", "all tags", "000", "000" , "getting all tags", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/extras/tags/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        alltags = {}
        for tag in data['results']:
            alltags[tag['name']] = tag['id']
        prettyllog("netbox", "get", "all tags", "000", r.status_code , "all tags found", severity="INFO")
        return alltags
    else:
        prettyllog("netbox", "get", "all tags", "000", r.status_code , "unable to get all tags", severity="ERROR")
        return False

def update_virtual_server(serverdetails, env , netboxdata):
    prettyllog("netbox", "update", "virtual server", serverdetails['hostName'], "000" , "updating virtual server", severity="INFO")
    if not serverdetails['hostName']:
        prettyllog("netbox", "update", "virtual server", "unknown", "000" , "no hostname", severity="ERROR")
        return False
    serverid = get_virtual_server_id(serverdetails['hostName'], env)
    if not serverid:
        prettyllog("netbox", "update", "virtual server", serverdetails['hostName'], "000" , "server not found", severity="ERROR")
        return False
    ipaddressid = get_ip_address_id(serverdetails['ipAddress'], env)
    if not ipaddressid:
        create_ip_address(serverdetails['ipAddress'], env, netboxdata)  
        ipaddressid = get_ip_address_id(serverdetails['ipAddress'], env)
    vmpathname = serverdetails['vmPathName']
    if "GODNOR" in vmpathname:
        clustergroup = "GOD"
        clustername = "GODNOR" 
    elif "GODSYD" in vmpathname:
        clustergroup = "GOD"
        clustername = "GODSYD"
    elif "SKS" in vmpathname:
        clustergroup = "SKS"
        clustername = "SKS"
    else:
        clustergroup = "OTHER"
        clustername = "OTHER"
    clusterid = get_cluster_id(clustername, env)
    alltags = get_all_tags(env)
    mytags = get_virtual_server_tags(serverid, env)
    found = False
    try:
        myvmwareid = alltags["vmware"]
    except:
        myvmwareid = False
    if not myvmwareid:
        myvmwareid = create_tag("vmware", env)
        alltags = get_all_tags(env)
    for mytag in mytags:
        if alltags[mytag] == "vmware":
            found = True
    if found == False:
        mytags.append(alltags["vmware"])

    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/virtual-machines/" + str(serverid) + "/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    data = {
        "name": serverdetails['hostName'],
        #strip all wovel characters and replace spaces with -
        "slug": serverdetails['hostName'].lower().replace(" ", "-"),
        "description": "Created by MANISBLE",
        "primary_ip4": ipaddressid,
        "memory": serverdetails['memory'],
        "vcpus": serverdetails['numCpu'],
        "site":  get_site_id("Aarhus Universitetshospital", env),
        "cluster": clusterid,
        "comments": str(serverdetails),
        "tags": mytags
        }
    r = requests.patch(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 201:
        data = r.json()
        prettyllog("netbox", "update", "virtual server", serverdetails['hostName'], r.status_code , "virtual server created", severity="CHANGE")
        return data['id']
    else:
        if r.status_code == 400:
            prettyllog("netbox", "update", "virtual server", serverdetails['hostName'], r.status_code , "virtual server exists", severity="INFO")
            return True
        prettyllog("netbox", "update", "virtual server", serverdetails['hostName'], r.status_code , "unable to create virtual server", severity="ERROR")
        pprint.pprint(r.content)
        return False
    
def create_interface(serverdetails, env, netboxdata):
    prettyllog("netbox", "create", "interface", serverdetails['hostName'], "000" , "creating interface", severity="INFO")
    if not serverdetails['hostName']:
        prettyllog("netbox", "create", "interface", "unknown", "000" , "no hostname", severity="ERROR")
        return False
    serverid = get_virtual_server_id(serverdetails['hostName'], env)
    if not serverid:
        prettyllog("netbox", "create", "interface", serverdetails['hostName'], "000" , "server not found", severity="ERROR")
        return False
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/interfaces/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    data = {
        "virtual_machine": serverid,
        "name": "primary",
        "enabled": True,
        "mac_address": "00:00:00:00:00:00",
        "mtu": 1500,
        "mode": "access",
        "description": "Created by MANISBLE"
        }
    r = requests.post(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    my_interface = get_interface_id(serverid, env)
    prettyllog("netbox", "create", "interface", serverdetails['hostName'], "000" , "interface id: " + str(my_interface), severity="INFO")
    if not my_interface:
        prettyllog("netbox", "create", "interface", serverdetails['hostName'], "000" , "interface not found", severity="ERROR")
        create_interface(serverdetails, env, netboxdata)    
        my_interface = get_interface_id(serverid, env)
    
    assign_ip_address(serverdetails['ipAddress'], serverdetails, env, netboxdata)

    


    if r.status_code == 201:
        
        data = r.json()
        prettyllog("netbox", "create", "interface", serverdetails['hostName'], r.status_code , "interface created", severity="CHANGE")
        return data['id']
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "interface", serverdetails['hostName'], r.status_code , "interface exists", severity="INFO")
            return True
        prettyllog("netbox", "create", "interface", serverdetails['hostName'], r.status_code , "unable to create interface", severity="ERROR")
        return False
    
def get_interface_id(serverid, env):
    prettyllog("netbox", "get", "interface id", serverid, "000" , "getting interface id", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/interfaces/?virtual_machine_id=" + str(serverid)
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        if len(data['results']) == 1:
            prettyllog("netbox", "get", "interface id", serverid, r.status_code , "interface id found", severity="INFO")
            return data['results'][0]['id']
        else:
            prettyllog("netbox", "get", "interface id", serverid, r.status_code , "interface id not found", severity="INFO")
            return False
    else:
        prettyllog("netbox", "get", "interface id", serverid, r.status_code , "unable to get interface id", severity="ERROR")
        return False
    
def assign_ip_address(ipaddress, serverdetails, env, netboxdata):
    #http://exrhel0494.it.rm.dk:8080/ipam/ip-addresses/add/?virtual_machine=846&vminterface=6&return_url=/virtualization/interfaces/6/

    prettyllog("netbox", "assign", "ip address", ipaddress, "000" , "assigning ip address", severity="INFO")
    ipaddressid = get_ip_address_id(ipaddress, env)
    if not ipaddressid:
        prettyllog("netbox", "assign", "ip address", "unknown", "000" , "no ip address", severity="ERROR")
        return False
    server_id = get_virtual_server_id(serverdetails['hostName'], env)
    if not server_id:
        prettyllog("netbox", "assign", "ip address", ipaddress, "000" , "server not found", severity="ERROR")
        return False
    interface_id = get_interface_id(server_id, env)
    if not interface_id:
        create_interface(serverdetails, env, netboxdata)
        interface_id = get_interface_id(server_id, env)
        prettyllog("netbox", "assign", "ip address", ipaddress, "000" , "interface not found", severity="ERROR")
        return False

    url = env['MANISBLE_NETBOX_URL'] + "/api/ipam/ip-addresses/" + str(ipaddressid) + "/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    data = {
            "assigned_object_type": "virtualization.vminterface",
            "assigned_object_id": interface_id,
        }
    r = requests.patch(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 201:
        data = r.json()
        prettyllog("netbox", "assign", "ip address", ipaddress, r.status_code , "ip address assigned", severity="CHANGE")
        return data['id']
    else:

        if r.status_code == 400:
            prettyllog("netbox", "assign", "ip address", ipaddress, r.status_code , "ip address exists", severity="INFO")
            return True
        prettyllog("netbox", "assign", "ip address", ipaddress, r.status_code , "unable to assign ip address", severity="ERROR")
        return False

def create_virtual_server(serverdetails, env, netboxdata):
    if not serverdetails['hostName']:
        prettyllog("netbox", "create", "virtual server", "unknown", "000" , "no hostname", severity="ERROR")
        return False
    virtual_server_id = get_virtual_server(serverdetails['hostName'], env)
    if virtual_server_id:
        prettyllog("netbox", "create", "virtual server", serverdetails['hostName'], "000" , "virtual server exists", severity="INFO")
        update_virtual_server(serverdetails, env, netboxdata)
        return True
    prettyllog("netbox", "create", "virtual server", serverdetails['hostName'], "000" , "creating virtual server", severity="INFO") 

    vmpathname = serverdetails['vmPathName']
    if "GODNOR" in vmpathname:
        clustergroup = "GOD"
        clustername = "GODNOR" 
    elif "GODSYD" in vmpathname:
        clustergroup = "GOD"
        clustername = "GODSYD"
    elif "SKS" in vmpathname:
        clustergroup = "SKS"
        clustername = "SKS"
    else:
        clustergroup = "OTHER"
        clustername = "OTHER"
#{'guestFullName': 'Red Hat Enterprise Linux 8 (64-bit)',
# 'guestId': 'rhel8_64Guest',
#'hostName': 'exrhel0537.it.rm.dk',
# 'hwVersion': 'vmx-19',
# 'ipAddress': '10.141.32.157',
# 'memory': 16384,
# 'numCpu': 4,
# 'numEthernetCards': 1,
#'numVirtualDisks': 3,
# 'overallStatus': 'green',
# 'toolsRunningStatus': 'guestToolsRunning',
# 'toolsStatus': 'toolsOk',
# 'toolsVersionStatus': 'guestToolsUnmanaged',
# 'toolsVersionStatus2': 'guestToolsUnmanaged',
# 'uuid': '420b9c5d-cc12-7b1b-62f0-2d3a85d0a9e8',
# 'vmPathName': '[GODSYD-SAN040_SYD-PROD-02_001] EXRHEL0537/EXRHEL0537.vmx'}



    pprint.pprint(serverdetails)
    guestFullName = serverdetails['guestFullName']
    guestId = serverdetails['guestId']
    hostName = serverdetails['hostName']
    hwVersion = serverdetails['hwVersion']
    ipAddress = serverdetails['ipAddress']
    memory = serverdetails['memory']
    numCpu = serverdetails['numCpu']
    numEthernetCards = serverdetails['numEthernetCards']
    numVirtualDisks = serverdetails['numVirtualDisks']
    overallStatus = serverdetails['overallStatus']
    toolsRunningStatus = serverdetails['toolsRunningStatus']
    toolsStatus = serverdetails['toolsStatus']
    toolsVersionStatus = serverdetails['toolsVersionStatus']
    toolsVersionStatus2 = serverdetails['toolsVersionStatus2']
    uuid = serverdetails['uuid']
    vmPathName = serverdetails['vmPathName']

    prettyllog("netbox", "create", "virtual server", serverdetails['hostName'], "000" , "creating cluster", severity="INFO")
    create_cluster(clustername, clustergroup, env, netboxdata)
    create_ip_address(ipAddress, env, netboxdata)
    
    
    clusterid = get_cluster_id(clustername, env)
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/virtual-machines/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
    data = {
        "name": serverdetails['hostName'],
        #strip all wovel characters and replace spaces with -
        "slug": serverdetails['hostName'].lower().replace(" ", "-"),
        "description": "Created by MANISBLE",
        'memory': memory,
        'vcpus': numCpu,



        "site":  get_site_id("Aarhus Universitetshospital", env),
        "cluster": clusterid
    }
    r = requests.post(url, headers=headers, json=data, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 201:
        data = r.json()
        prettyllog("netbox", "create", "virtual server", serverdetails['hostName'], r.status_code , "virtual server created", severity="CHANGE")
        prettyllog("netbox", "create", "virtual server", serverdetails['hostName'], "000" , "creating interface", severity="INFO")
        create_interface(serverdetails, env, netboxdata)
        prettyllog("netbox", "create", "virtual server", serverdetails['hostName'], "000" , "assigning ip address", severity="INFO")
        assign_ip_address(serverdetails['ipAddress'], serverdetails, env, netboxdata)
        return data['id']
    else:
        if r.status_code == 400:
            prettyllog("netbox", "create", "virtual server", serverdetails['hostName'], r.status_code , "virtual server exists", severity="INFO")
            update_virtual_server(serverdetails, env, netboxdata)
            return True
        prettyllog("netbox", "create", "virtual server", serverdetails['hostName'], r.status_code , "unable to create tenant_group", severity="ERROR")
        return False
    


import requests
import os   
import pprint

def checkenv():
    myenv = {}
    myenv['VAULT_ADDR'] = os.environ.get('MANISBLE_VAULT_ADDR')   
    myenv['VAULT_TOKEN'] = os.environ.get('MANISBLE_VAULT_TOKEN')
    myenv['VAULT_NAMESPACE'] = os.environ.get('MANISBLE_VAULT_NAMESPACE')
    myenv['VAULT_FORMAT'] = os.environ.get('MANISBLE_VAULT_FORMAT')

    if myenv['VAULT_FORMAT'] == None:
        myenv['VAULT_FORMAT'] = "json"

    #Check if we can connect to vault
    
    r = requests.get(myenv['VAULT_ADDR'], headers={"X-Vault-Token": myenv['VAULT_TOKEN']})
    if r.status_code != 200:
            print(r.status_code)
            print(r.content)
            print("Could not connect to vault, status code: " + str(r.status_code))
            exit(1)
    #check if we have a .ssh folder
    if not os.path.exists(os.path.expanduser("~/.ssh")):
        print("Could not find ~/.ssh folder")
        print("create it with mkdir ~/.ssh")
        os.mkdir(os.path.expanduser("~/.ssh"))
        #se access rights
        os.chmod(os.path.expanduser("~/.ssh"), 0o700)
    #check if we have a known_hosts file
    if not os.path.exists(os.path.expanduser("~/.ssh/known_hosts")):
        print("Could not find ~/.ssh/known_hosts file")
        print("create it with touch ~/.ssh/known_hosts")
        os.system("touch ~/.ssh/known_hosts")
        #se access rights
        os.chmod(os.path.expanduser("~/.ssh/known_hosts"), 0o600)
    myenv['SSH_KNOWN_HOSTS'] = os.path.expanduser("~/.ssh/known_hosts")
    return myenv





def signssh():
    myenv = checkenv()
    pprint.pprint(myenv)


from . import ssh
import argparse
from ..common import prettyllog


def main():
    parser = argparse.ArgumentParser(description="Keep manisble and automate ssh", usage="manisble_ssh <action> \n\n \
               \
               version : 0.0.1 manisble_ssh  \n\
               actions:                  \n\
               signssh                   \n\
               \
               2023 Knowit Miracle\
               ")
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup jenkis')
    args = parser.parse_args()
    ready = False

    if args.action[0] == "signssh":
        ssh.signssh()


import requests
import subprocess
import json
import os
import redis
import sys
import time 

netbox_url = os.environ.get('NETBOX_URL')
netbox_token = os.environ.get('NETBOX_TOKEN')

NETBOX_URL = os.getenv("NETBOX_API_URL")
NETBOX_TOKEN = os.getenv("NETBOX_API_TOKEN")


def am_I_virtual_machine():
    return os.path.exists("/proc/1/environ")

def get_virtual_machines():
    returnvms = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/virtualization/virtual-machines/")
    response = requests.get(url, headers=headers)
    vms = response.json()
    for vm in vms["results"]:
        try:
            if returnvms[vm["name"]]:
                print("Duplicate vm name")
        except:
           returnvms[vm["name"]] = vm["id"]
    return returnvms

def get_virtual_machine(id):
    returnvm = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/virtualization/virtual-machines/" + str(id))
    response = requests.get(url, headers=headers)
    vm = response.json()
    return vm    


def get_local_context_data():
    if am_I_virtual_machine():
        # we are running in a virtual machine
        # get the local context data from netbox
        vms = get_virtual_machines()
        vmdata = get_virtual_machine(vms[os.environ.get("HOSTNAME")])
        return vmdata["local_context_data"]

def ansible_inventory(args):
    clusters = get_clusters()
    vms = get_virtual_machines()

    vm_data = []
    for vm in vms:
        vm_entry = {
            "name": vm["name"],
            "cluster": vm["cluster"]["name"] if vm.get("cluster") else "N/A",
            "disk_gb": vm["disk"],
            "cpu": vm["vcpus"],
            "memory_mb": vm["memory"],
            "local_context_data": vm["local_context_data"]
        }
        vm_data.append(vm_entry)

    data = {
        "_meta": {
            "hostvars": {}
        },
        "all": {
            "children": ["clusters"]
        },
        "clusters": {
            "hosts": [cluster["name"] for cluster in clusters]
        }
    }

    for vm in vms:
        if vm.get("cluster"):
            cluster_name = vm["cluster"]["name"]
            if cluster_name not in data:
                data[cluster_name] = {"hosts": []}
            data[cluster_name]["hosts"].append(vm["name"])
            data["_meta"]["hostvars"][vm["name"]] = {
                "disk_gb": vm["disk"],
                "cpu": vm["vcpus"],
                "memory_mb": vm["memory"]
            }

    print(yaml.dump(data))



def refresh(args):
 print(get_local_context_data)

 



from . import inabox
import argparse

def main():
    parser = argparse.ArgumentParser(description="Keep manisble and automate", usage="manisble <action> \n\n \
               \
               version : 0.1.2 BETA \n                                              \
               actions:\n                                                      \
               list        list inabox \n  \
               rancher     activate rancher in a box\n  \
               \
               2023 Knowit Miracle\
               ")
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup netbox')
    args = parser.parse_args()
    ready = False
    print("check if we are ready to go")

    if args.action[0] == "list":
        print("list inabox")
        inabox.list_inabox()
        
    if args.action[0] == "rancher":
        print("Start rancher in a boc")
        inabox.rancher_inabox()

    if args.action[0] == "ansible_role":
        print("Create ansible role directory structure")
        inabox.ansible_role_inabox()
        


    








# keep manisble and manage your vspere

from . import serve
import argparse

def main():
    parser = argparse.ArgumentParser(description="Keep manisble and con5entrate on your semaphores", usage="manisble_semaphore <action> \n\n \
\
version : 0.0.2 (semaphore)\n\
actions:\n\
serve      keep manisble and serve vspere\n\
\n\
\
2023 Knowit Miracle\
")
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup jenkis')
    args = parser.parse_args()
    ready = False

    if args.action[0] == "serve":
        serve.main()
        return 0
    


from ..common import prettyllog
import json
import os
import requests


def get_all_tags(env):
    prettyllog("netbox", "get", "all tags", "000", "000" , "getting all tags", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/extras/tags/"
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        alltags = {}
        for tag in data['results']:
            alltags[tag['name']] = tag['id']
        prettyllog("netbox", "get", "all tags", "000", r.status_code , "all tags found", severity="INFO")
        return alltags
    else:
        prettyllog("netbox", "get", "all tags", "000", r.status_code , "unable to get all tags", severity="ERROR")
        return False
    
def get_virtual_server_tags(serverid, env):
    prettyllog("netbox", "get", "virtual server tags", serverid, "000" , "getting virtual server tags", severity="INFO")
    url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/virtual-machines/" + str(serverid)
    headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
               'Accept': 'application/json',
               'Content-Type': 'application/json'
            }
    mytags = []
    r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
    if r.status_code == 200:
        data = r.json()
        for tag in data['tags']:
            mytags.append(tag['name'])
        prettyllog("netbox", "get", "virtual server tags", serverid, r.status_code , "virtual server tags found", severity="INFO")
        return mytags
    else:
        prettyllog("netbox", "get", "virtual server tags", serverid, r.status_code , "unable to get virtual server tags", severity="ERROR")
        return mytags      
    
def get_virtual_server_id(servername, env):
        prettyllog("netbox", "get", "virtual server id", servername, "000" , "getting virtual server id", severity="INFO")
        url = env['MANISBLE_NETBOX_URL'] + "/api/virtualization/virtual-machines/?name=" + servername
        headers = {'Authorization': 'Token ' + env['MANISBLE_NETBOX_TOKEN'],
                   'Accept': 'application/json',
                   'Content-Type': 'application/json'
                }
        r = requests.get(url, headers=headers, verify=env['MANISBLE_NETBOX_SSL'])
        if r.status_code == 200:
                data = r.json()
                if data['count'] == 1:
                        prettyllog("netbox", "get", "virtual server id", servername, r.status_code , "virtual server id found", severity="INFO")
                        return data['results'][0]['id']
                else:
                        prettyllog("netbox", "get", "virtual server id", servername, r.status_code , "unable to get virtual server id", severity="ERROR")
                return False
        else:
                prettyllog("netbox", "get", "virtual server id", servername, r.status_code , "unable to get virtual server id", severity="ERROR")
                return False
        
def setrandomcolor():
        ###############################
        return "00ff00"


def update_subprojects(myenv):
        prettyllog("netbox", "init", "subprojects", "manisble" "ok", "000" , "updating subprojects", severity="INFO")
        for subproject in myenv['subproject']:
                print(subproject)
                for myhost in myenv['subproject'][subproject]['hosts']:
                        myserverid = get_virtual_server_id(myhost, myenv)
                        if myserverid:
                                alltags = get_all_tags(myenv)
                                try:
                                        mysubprojecttagid = alltags[subproject]
                                except:
                                        mysubprojecttagid = False
                                if not mysubprojecttagid:
                                        print("adding tag %s to netbox" % subproject)
                                        url = myenv['MANISBLE_NETBOX_URL'] + "/api/extras/tags/"
                                        headers = {'Authorization': 'Token ' + myenv['MANISBLE_NETBOX_TOKEN'],
                                                   'Accept': 'application/json',
                                                   'Content-Type': 'application/json'
                                                }
                                        data = {
                                                "name": subproject,
                                                "slug": subproject.lower(),
                                                "color": setrandomcolor()
                                                }
                                        r = requests.post(url, headers=headers, data=json.dumps(data), verify=myenv['MANISBLE_NETBOX_SSL'])
                                        if r.status_code == 201:
                                                print("tag %s added to netbox" % subproject)
                                        else:
                                                print("unable to add tag %s to netbox" % subproject)
                                else:
                                        print("tag %s already exists in netbox" % subproject)
                                foundmysubprojecttag = False

                                mytags = get_virtual_server_tags(myserverid, myenv)
                                for tag in mytags:
                                        if tag == subproject:
                                                foundmysubprojecttag = True
                                if foundmysubprojecttag:
                                        print("server %s already has tag %s" % (myhost, subproject))
                                else:
                                        mytagids = []
                                        for tag in mytags:
                                                mytagids.append(alltags[tag])
                                        mytagids.append(mysubprojecttagid)
                                        print("server %s does not have tag %s" % (myhost, subproject))
                                        url = myenv['MANISBLE_NETBOX_URL'] + "/api/virtualization/virtual-machines/" + str(myserverid) + "/"
                                        headers = {'Authorization': 'Token ' + myenv['MANISBLE_NETBOX_TOKEN'],
                                                   'Accept': 'application/json',
                                                   'Content-Type': 'application/json'
                                                }
                                        data = {
                                                "tags": mytagids
                                                }
                                        r = requests.patch(url, headers=headers, data=json.dumps(data), verify=myenv['MANISBLE_NETBOX_SSL'])
                                        if r.status_code == 200:
                                                print("tag %s added to server %s" % (subproject, myhost))
                                        else:
                                                print(r.content)
                                                print("unable to add tag %s to server %s" % (subproject, myhost))


                        else:
                                print("server %s not found" % myhost)
        return True




                        


        return myenv



import redis
import json
import pprint   
import time
from ..common import prettyllog
from .netbox_server import create_virtual_server
from .common import get_env
import os



def refresh_netbox_from_redis(myenv, netboxdata):
    r = redis.Redis(host='localhost', port=6379, db=0)
    knownservers = {}
    knownlinuxservers = {}
    orphanservers = []
    knownserverkeys = r.keys("manisble:vmware:*:known")
    knownlinuxserverkeys = r.keys("manisble:vmware:*:known:linux")
    for key in knownlinuxserverkeys:
        key = key.decode("utf-8")
        server = key.split(":")[2]
        value = r.get(key)
        value = value.decode("utf-8")
        age = time.time() - float(value)
        ageindays = int(age / 86400)
        ageinhours = int(age / 3600)
        ageinminutes = int(age / 60)
        if ageindays > 1:
            prettyllog("netbox", "get", "server", key, ageindays , "Date is older than one day", severity="INFO")
            orphanservers.append(server)
        else:
            if ageindays < 1 and ageinhours > 1:
                prettyllog("netbox", "get", "server", key, ageinhours , "Data is aging (Hours)", severity="INFO")
            else:
                prettyllog("netbox", "get", "server", key, ageinminutes , "data is fresh (Minutes)", severity="INFO")
            #
            detailkey = "manisble:vmware:" + server + ":details"
            if r.exists(detailkey):
              detailvalue = r.get(detailkey)
              decodeddetailvalue = detailvalue.decode("utf-8").replace("'", '"')
              knownlinuxservers[server] = detailvalue.decode("utf-8")
              detailjson = json.loads(decodeddetailvalue)   
              pprint.pprint(detailjson)
              create_virtual_server(detailjson, myenv, netboxdata)
            else:
                prettyllog("netbox", "get", "server", key, "000" , "No details found", severity="ERROR")
                orphanservers.append(server)
    print("orphanservers: %s" % len(orphanservers))
    print("knownservers:  %s" % len(knownservers))
    return True


